﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using NathanScott_Lib;
using System.Xml.Serialization;
using System.IO;
using System.Runtime.Serialization;

namespace FinalProject
{
    public partial class FinalProjectMainForm : Form
    {
        #region Members

        // viewports
        private HeroControlHandler mViewHero;
        private CameraControlHandler mViewWorld;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="FinalProjectMainForm"/> class.
        /// </summary>
        public FinalProjectMainForm()
        {
            InitializeComponent();

            mViewHero = new HeroControlHandler(Program.getModel());
            mViewWorld = new CameraControlHandler(Program.getModel());

            //If we are in release then hide the editor guis
            #if !DEBUG
                _btnObjectEditor.Visible = false;
                _ddObjectEditor.Visible = false;
            #endif

            _btnBag.BackgroundImage = Properties.Resources.Bag;
            _lblAir.BackgroundImage = Properties.Resources.HoldingBreath;
            _lblAir.BackgroundImageLayout = ImageLayout.Zoom;
            _lblMoney.BackgroundImage = Properties.Resources.Money;

            Model model = Program.getModel();
            if (model != null && model.getHero() != null)
            {
                updateSceneDropDown();
                updateEquippedItem(model.getHero().getEquippedItem());
            }
            //_btnInventory.BackgroundImage = Properties.Resources.AirTank;
        }

        public void updateModel()
        {
            Model model = Program.getModel();
            if (model != null)
            {
                setupViews();
                updateSceneDropDown();
                mViewHero.setModel(model);
                mViewWorld.setModel(model);
                updateEquippedItem(model.getHero().getEquippedItem());
            }
        }

        #endregion

        #region Public Methods

        public void play()
        {
            mTimer.Start();
        }

        public void pause()
        {
            mTimer.Stop();
        }

        public void updateSceneDropDown()
        {
#if DEBUG
            object obj = _ddObjectEditor.SelectedItem;

            _ddObjectEditor.Items.Clear();

            Model model = Program.getModel();
            if (model == null)
                return;

            UWB_SceneNode root = model.getSceneRoot();
            for (int i = 0; i < root.numChildren(); i++)
            {
                UWB_SceneNode node = root.getChildNode(i);
                _ddObjectEditor.Items.Add(node);
            }

            if (obj != null && _ddObjectEditor.Items.Contains(obj))
                _ddObjectEditor.SelectedItem = obj;
            else if(_ddObjectEditor.Items.Contains(model.getHero()))
                _ddObjectEditor.SelectedItem = model.getHero();
            else
                _ddObjectEditor.SelectedIndex = 0;
#endif
        }

        public void updateEquippedItem(Item item)
        {
            if (item != null)
            {
                _btnInventory.BackgroundImageLayout = ImageLayout.Zoom;
                _btnInventory.BackgroundImage = item.PreviewImage;
                _btnInventory.Text = item.getInventoryText();
                _tbInventory.Text = item.getName();
                _btnInventory.Tag = item;
            }
            else
            {
                _btnInventory.BackgroundImageLayout = ImageLayout.None;
                _tbInventory.Text = "";
                _btnInventory.Tag = "";
            }
        }

        public void updateGameDifficulty()
        {
            Model model = Program.getModel();
            if (model == null)
                return;

            if (model.GameDifficulty == eGameDifficulty.Hard)
            {
                _lblDiveComputer.Text = "Dive Computer";
                _tbDiveTimeLeft.Visible = false;
                _lblDiveTimeLeft.Visible = false;
            }
            else
            {
                _lblDiveComputer.Text = "Dive Computer Pro";
                _tbDiveTimeLeft.Visible = true;
                _lblDiveTimeLeft.Visible = true;
            }
        }

        public void updateUI()
        {
            Model model = Program.getModel();
            if (model == null)
                return;

            //Update the UI Draw area group
            int width = 0;
            int height = 0;
            mViewWorld.GetDeviceSize(ref width, ref height);
            mViewHero.GetDeviceSize(ref width, ref height);
            _tbMoney.Text = model.getHeroMoney().ToString("$0.00");
            _tbCollectableItems.Text = model.getHeroBagCount() + "/" + model.getHeroBagSize();
            _barAir.Maximum = Convert.ToInt32(Math.Round(model.getHeroAirMax()));
            _barAir.SetBounds(_barAir.Location.X, _barAir.Location.Y, _barAir.Maximum, _barAir.Size.Height);
            _barAir.Value = Convert.ToInt32(Math.Round(model.getHeroAir()));
            _tbCurrentDepth.Text = model.getHeroDiveDepth().ToString("0.00");
            _tbMaxDiveDepth.Text = model.getHeroMaxDiveDepth().ToString("0.00");

            if (_btnInventory.Tag != null && _btnInventory.Tag is Item)
            {
                Item item = _btnInventory.Tag as Item;
                _btnInventory.Text = item.getInventoryText();
            }

            if (model.getHero() != null && model.getHero().getAirTank() != null)
            {
                _lblAir.BackgroundImage = model.getHero().getAirTank().PreviewImage;
            }
            else
            {
                _lblAir.BackgroundImage = Properties.Resources.HoldingBreath;
            }

            TimeSpan timeSpan = model.getHeroDiveTime();
            string time = timeSpan.Hours.ToString() + ":";
            time += timeSpan.Minutes.ToString() + ":";
            time += timeSpan.Seconds.ToString() + ":";
            time += timeSpan.Milliseconds.ToString();
            _tbDiveTime.Text = time;

            timeSpan = model.getHerDiveTimeLeft();
            time = timeSpan.Hours.ToString() + ":";
            time += timeSpan.Minutes.ToString() + ":";
            time += timeSpan.Seconds.ToString() + ":";
            time += timeSpan.Milliseconds.ToString();
            _tbDiveTimeLeft.Text = time;
        }

        public void openHelpMenu(string keyword)
        {
            if(keyword == string.Empty)
                Help.ShowHelp(this, mHelpProvider.HelpNamespace);
            else
                Help.ShowHelp(this, mHelpProvider.HelpNamespace, HelpNavigator.KeywordIndex, keyword);
        }

        #endregion

        #region Event Service Methods

        /// <summary>
        /// Handles the MouseMove event of the Mp4Form control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        private void FinalProjectMainForm_MouseMove(object sender, MouseEventArgs e)
        {
            Model model = Program.getModel();
            if (model != null)
            {
                System.Drawing.Point pos = this.PointToClient(Cursor.Position);

                model.setMouseInfo("Main Dialog", new Vector2(pos.X, pos.Y), Vector3.Zero, Vector3.Zero);
            }
        }

        /// <summary>
        /// Handles the MouseLeave event of the Mp4Form control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void FinalProjectMainForm_MouseLeave(object sender, EventArgs e)
        {
            Model model = Program.getModel();
            if (model != null)
                model.setMouseInfo("Unknown", Vector2.Zero, Vector3.Zero, Vector3.Zero);
        }

        /// <summary>
        /// Handles the Load event of the Mp4Form control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void FinalProjectMainForm_Load(object sender, EventArgs e)
        {
            setupViews();

            //Hook up where our status/echo text will go
            N_Utility.addEchoTextBox(mStatusText);

            //Start our timer
            mTimer.Start();

            N_Utility.echoToStatusArea("Form Load Complete");
        }

        private void setupViews()
        {
            Model model = Program.getModel();
            if (model == null)
                return;

            //mGameControl.setModel(model);

            mViewWorld.Initialize(mPlaceholderWorld);
            mViewHero.Initialize(mPlaceholderHero);

            //Link our world bounds with our large viewport
            UWB_BoundingBox worldBoundLarge = model.getWorldBounds();
            mViewWorld.setWCWindow(ref worldBoundLarge);

            //Set the small zoom box to have the same height (entire head)
            //but change our width so that the aspect ratio stays the same as our small window
            float height = model.getZoomedInViewBox().getHeight();
            float width = model.getZoomedInViewBox().getWidth();
            float ratio = mViewHero.getAspectRatio();
            width = height * ratio;
            model.initZoomViewDimensions(width, height);

            //Update the gui to match our model
            //mGameControl.ShootRate = model.getShootRate();

            //Link our zoom view box to our small window
            UWB_BoundingBox worldBoundSmall = model.getZoomedInViewBox();
            mViewHero.setWCWindow(ref worldBoundSmall);
        }

        /// <summary>
        /// Handles the Tick event of the mTimer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mTimer_Tick(object sender, EventArgs e)
        {
            Model model = Program.getModel();
            if (model == null)
                return;

            //Handle in user keyboard events
            model.updateInput();

            //Update the simulation (collision, movement...)
            model.updateSimulation();

            // update GameControl panel.
            //int numHitsGoal, numHitsNet;
            //model.getNumHits(out numHitsGoal, out numHitsNet);
            //mGameControl.setNumHits(numHitsGoal, numHitsNet);

            //Draw Viewports
            mViewWorld.DrawGraphics();
            mViewHero.DrawGraphics();

            updateUI();

            //Update the status area showing the number of shooting balls
            //UWB_Utility.echoToStatusArea(model.getNumShootingBalls().ToString("0"), 1);
        }

        private void _btnInventory_Click(object sender, EventArgs e)
        {
            Model model = Program.getModel();
            if (model != null && model.getHero() != null)
            {
                InventoryWindow inventory = new InventoryWindow();
                mHelpProvider.SetHelpNavigator(inventory, HelpNavigator.KeywordIndex);
                mHelpProvider.SetHelpKeyword(inventory, "Inventory");
                inventory.InventoryCloseCallback += new InventoryWindow.InventoryCloseEventHanlder(onOtherWindowClose);
                inventory.ini(model.getHero());
                inventory.Show();

                Program.pause();
            }
        }

        private void _btnBag_Click(object sender, EventArgs e)
        {
            Model model = Program.getModel();
            if (model != null && model.getHero() != null)
            {
                BagWindow bag = new BagWindow();
                mHelpProvider.SetHelpNavigator(bag, HelpNavigator.KeywordIndex);
                mHelpProvider.SetHelpKeyword(bag, "Collectable");
                bag.InventoryCloseCallback += new BagWindow.BagCloseEventHanlder(onOtherWindowClose);
                bag.ini(model.getHero());
                bag.Show();

                Program.pause();
            }
        }

        private void onOtherWindowClose()
        {
            Program.play();
        }

        /// <summary>
        /// Handles the Click event of the _bnObjectEditor control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void _btnObjectEditor_Click(object sender, EventArgs e)
        {
            Program.openObjectEditor(_ddObjectEditor.SelectedItem as UWB_SceneNode);
        }

        private void newGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.DoNewGame();
        }

        private void loadGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.pause();
            if(mLoadGameDialog.ShowDialog() == DialogResult.OK)
                Program.DoLoadGame(mLoadGameDialog.FileName);
            Program.play();
        }

        private void saveGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.pause();
            if (mSaveGameDialog.ShowDialog() == DialogResult.OK)
                Program.DoSaveGame(mSaveGameDialog.FileName);
            Program.play();
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void gameHelpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openHelpMenu(string.Empty);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutWindow aboutWindow = new AboutWindow();
            aboutWindow.Show();
        }

        #endregion
    }
}
