﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib13;
using NathanScott_Lib;
using FinalProject;
using System.Xml.Serialization;
using System.Runtime.Serialization;

namespace FinalProject
{
    [Serializable]
    public class Model : TwoViewModel
    {
        #region Members
        
        /// <summary>
        /// Holds the game story
        /// </summary>
        private string mStory = "";

        private eGameDifficulty mGameDifficulty = eGameDifficulty.Easy;

        /// <summary>
        /// For left clicking and dragging hero around, need this.
        /// </summary>
        private Vector3 mOriginalHeroLocation = Vector3.Zero;

        // for tracking keyboard input
        private KeyboardState mOldKeyboardState;

        // for storing all world objects
        private List<SceneObject> mWorldObjects;

        //Store all our projectiles
        private List<SceneObject> mProjectiles;

        //Store all our collectibles
        private List<Item> mCollectibles;

        //Special objects we will use a lot
        private Hero mHero;
        private Boat mBoat;
        private WinGameItem mWingameItem;
        private UWB_PrimitiveList mBackDrop;

        // water bounds, for tracking when hero is trying to leave the water world..
        private UWB_BoundingBox mWaterBounds;

        // for moving hero back away from wall if he collides with a wall
        private Vector3 mPreviousHeroTranslation;
        private float mPreviousHeroRotationDegrees;

        #endregion

        #region Constructors

        /// <summary>
        /// Get a random story
        /// </summary>
        /// <returns>random story</returns>
        public static string getNewStory()
        {
            int value = N_Utility.GenerateRandomInt(0, 2);

            StringBuilder sb = new StringBuilder();

            switch(value)
            {
                case 0:
                    sb.AppendLine("You have lost everything in the housing crisis.");
                    sb.Append(Environment.NewLine);
                    break;
                case 1:
                    sb.AppendLine("Your wife left and took the kids, house, cars, money, clothes...even the ice cube trays.");
                    sb.Append(Environment.NewLine);
                    sb.AppendLine("The only items she left were your mask, wetsuit and flippers.  She never did like your diving hobby...");
                    sb.Append(Environment.NewLine);
                    sb.AppendLine("As if to prove that point, she took your best diving gear and tossed it overboard while riding the ferry out of town.");
                    sb.Append(Environment.NewLine);
                    break;
            }

            sb.Append("Left with only a mask, wetsuit, flippers, you will try to gain it all back amongst the overly-shark-infested-yet-oddly-gold-coin-laden waters off the coast of Carchaurum.");

            return sb.ToString();
        }

        public string getStory()
        {
            return mStory;
        }

        /// <summary>
        /// Get/set the game difficulty
        /// </summary>
        public eGameDifficulty GameDifficulty
        {
            get
            {
                return mGameDifficulty;
            }

            set
            {
                mGameDifficulty = value;
                if (mWingameItem != null)
                {
                    float amount = WinGameItem.getPriceBasedOnDifficulty(value);
                    mWingameItem.MinPrice = amount;
                    mWingameItem.MaxPrice = amount;
                }
                Program.updateGameDifficulty();
            }
        }

        public void LoadWorld()
        {
            mOldKeyboardState = Keyboard.GetState();

            mWaterBounds = new UWB_BoundingBox();
            mWaterBounds.setCorners(new Vector3(mWorldBounds.getMin().X, mWorldBounds.getMin().Y, mWorldBounds.getMin().Z),
                                    new Vector3(mWorldBounds.getMax().X, mWorldBounds.getMax().Y * .8f, mWorldBounds.getMax().Z));

            //Set up the scene
            mScene = new UWB_SceneNode("Scene");

            //Set up our backdrop
            mBackDrop = createBackDrop();

            UWB_SceneNode node = new UWB_SceneNode("Backdrop");
            node.setPrimitive(mBackDrop);
            mScene.insertChildNode(node);

            List<SceneObject> tempWorldObjects = mWorldObjects; // these don't have primitives yet, so we need to hold them here and re-create them and then add to the real one.
            mWorldObjects = new List<SceneObject>();

            // re-create and add world objects to scene
            if ((tempWorldObjects != null) && (tempWorldObjects.Count > 0))
            {
                foreach (SceneObject sceneObject in tempWorldObjects)
                {
                    // I'm still working out how to fix this in a more elegant way
                    if (sceneObject is Wall)
                    {
                        Wall tempWall = new Wall(((Wall)(sceneObject)).MinXML, ((Wall)(sceneObject)).LengthXML, ((Wall)(sceneObject)).WallTextureXML, ((Wall)(sceneObject)).Orientation);
                        tempWall.setVelocity(sceneObject.Velocity);
                        mWorldObjects.Add(tempWall);
                        addToScene(tempWall);
                    }
                    else if (sceneObject is Shark)
                    {
                        Shark tempShark = new Shark(((Shark)(sceneObject)).PivotXML, ((Shark)(sceneObject)).LengthXML);
                        tempShark.setVelocity(sceneObject.Velocity);
                        mWorldObjects.Add(tempShark);
                        addToScene(tempShark);
                    }
                }
            }

            List<Item> tempCollectibles = mCollectibles; // these don't have primitives yet, so we need to hold them here and re-create them and then add to the real one.
            mCollectibles = new List<Item>();

            // re-create and add world objects to scene
            if ((tempCollectibles != null) && (tempCollectibles.Count > 0))
            {
                foreach (Item item in tempCollectibles)
                {
                    // I'm still working out how to fix this in a more elegant way
                    if (item is Coin)
                    {
                        Coin coin = item as Coin;
                        Coin tempCoin = new Coin(coin.PivotXML, coin.RadiusXML, coin.TextureXML, coin.Condition, coin.Level);
                        mCollectibles.Add(tempCoin);
                        addToScene(tempCoin);
                    }
                }
            }

            // re-create and add boat to scene - shouldn't this also be in world objects??
            Boat tempBoat = new Boat(mBoat.PivotXML, mBoat.LengthXML);

            if ((mBoat.Inventory != null) && (mBoat.Inventory.getItemCount() > 0))
            {
                for (int i = 0; i < mBoat.Inventory.getItemCount(); i++)
                {
                    Item item = mBoat.Inventory.getItem(i);

                    if (item is AirTank)
                    {
                        AirTank tempAirTank = new AirTank(((AirTank)(item)).PivotXML, ((AirTank)(item)).TorsoHeightXML, ((AirTank)(item)).TorsoWidthXML);
                        tempBoat.Inventory.addItem(tempAirTank);
                    }
                    else if (item is Knife)
                    {
                        Knife tempKnife = new Knife(((Knife)(item)).PivotXML, ((Knife)(item)).LengthXML);
                        tempBoat.Inventory.addItem(tempKnife);
                    }
                    else if (item is SpearGun)
                    {
                        SpearGun tempSpearGun = new SpearGun(((SpearGun)(item)).PivotXML, ((SpearGun)(item)).LengthXML, ((SpearGun)(item)).HasSpearXML);
                        tempBoat.Inventory.addItem(tempSpearGun);
                    }
                    else if (item is WinGameItem)
                    {
                        WinGameItem tempWinGameItem = new WinGameItem(((WinGameItem)(item)).MinPrice, ((WinGameItem)(item)).MaxPrice);
                        tempBoat.Inventory.addItem(tempWinGameItem);
                        mWingameItem = tempWinGameItem;
                    }
                }
            }

            mBoat = tempBoat;
            addToScene(mBoat);

            // re-create and add hero to scene
            Hero tempHero = new Hero(mHero.name, mHero.location, mHero.torsoWidth, mHero.torsoHeight, mHero.armLength, mHero.armWidth, mHero.legLength, mHero.legWidth, mHero.headRadius);
            tempHero.setVelocity(mHero.Velocity);

            // set hero's money and stuff
            tempHero.Money = mHero.Money;
            tempHero.Air = mHero.Air;
            tempHero.AirMax = mHero.AirMax;
            tempHero.IsAlive = mHero.IsAlive;

            // set hero's position and everything appropriately
            UWB_XFormInfo tempXForm = tempHero.getXFormInfo();
            UWB_XFormInfo mHeroXForm = mHero.getXFormInfo();
            tempXForm.setPivot(mHeroXForm.getPivot());
            tempXForm.setRotationDegrees(mHero.Rotation);
            tempXForm.setScale(mHero.Scale);
            tempXForm.setTranslation(mHero.Position);
            tempHero.setXFormInfo(tempXForm);

            // re-create hero's inventory, and attach any items necessary
            tempHero.Inventory = new Inventory();

            if ((mHero.Inventory != null) && (mHero.Inventory.getItemCount() > 0))
            {
                for (int i = 0; i < mHero.Inventory.getItemCount(); i++)
                {
                    Item item = mHero.Inventory.getItem(i);

                    if (item is AirTank)
                    {
                        AirTank tempAirTank = new AirTank(((AirTank)item).PivotXML, ((AirTank)item).TorsoHeightXML, ((AirTank)item).TorsoWidthXML);
                        tempHero.Inventory.addItem(tempAirTank);

                        // attach air tank to scene graph
                        tempHero.changeAirTank(tempAirTank);
                    }
                    else if (item is Knife)
                    {
                        Knife tempKnife = new Knife(((Knife)(item)).PivotXML, ((Knife)(item)).LengthXML);
                        tempHero.Inventory.addItem(tempKnife);
                    }
                    else if (item is SpearGun)
                    {
                        SpearGun tempSpearGun = new SpearGun(((SpearGun)(item)).PivotXML, ((SpearGun)(item)).LengthXML, ((SpearGun)(item)).HasSpearXML);
                        tempSpearGun.SpearCount = ((SpearGun)(item)).SpearCount;
                        tempHero.Inventory.addItem(tempSpearGun);
                    }
                    else if (item is WinGameItem)
                    {
                        WinGameItem tempWinGameItem = new WinGameItem(((WinGameItem)(item)).MinPrice, ((WinGameItem)(item)).MaxPrice);
                        tempHero.Inventory.addItem(tempWinGameItem);
                        mWingameItem = tempWinGameItem;
                        tempHero.updateWinGame();
                    }
                }
            }

            // set hero's equipped item, if any
            if (mHero.EquippedItemXML != null)
            {
                Item lastItem = null;

                // must use the same item from the inventory.
                Item itemToEquip = tempHero.Inventory.getItemOfSameType(mHero.EquippedItemXML);
                tempHero.setEquippedItem(itemToEquip, ref lastItem);
            }

            // re-create hero's bag
            tempHero.Bag = new Inventory();

            if ((mHero.Bag != null) && (mHero.Bag.getItemCount() > 0))
            {
                for (int i = 0; i < mHero.Bag.getItemCount(); i++)
                {
                    Item item = mHero.Bag.getItem(i);

                    if (item is Coin)
                    {
                        Coin tempCoin = new Coin(((Coin)item).PivotXML, ((Coin)item).RadiusXML, ((Coin)item).TextureXML, ((Coin)item).Condition, ((Coin)item).Level);
                        tempHero.Bag.addItem(tempCoin);
                        addToScene(tempCoin);
                    }
                }
            }

            mHero = tempHero;
            addToScene(mHero);

            List<SceneObject> tempProjectiles = mProjectiles; // these don't have primitives yet, so we need to hold them here and re-create them and then add to the real one.
            mProjectiles = new List<SceneObject>();

            // re-create and add projectiles to scene
            if ((tempProjectiles != null) && (tempProjectiles.Count > 0))
            {
                foreach (SceneObject sceneObject in tempProjectiles)
                {
                    // I'm still working out how to fix this in a more elegant way
                    if (sceneObject is Spear)
                    {
                        UWB_XFormInfo spearXFormInfo = new UWB_XFormInfo();
                        spearXFormInfo.setRotationDegrees(((Spear)(sceneObject)).Rotation);
                        spearXFormInfo.setScale(((Spear)(sceneObject)).Scale);
                        spearXFormInfo.setTranslation(((Spear)(sceneObject)).Position);
                        Spear tempSpear = new Spear(((Spear)(sceneObject)).Position, ((Spear)(sceneObject)).SpearLength, spearXFormInfo);
                        tempSpear.setVelocity(sceneObject.Velocity);
                        tempSpear.SetSpearColor(((Spear)(sceneObject)).SpearColor);
                        mProjectiles.Add(tempSpear);
                        addToScene(tempSpear);
                    }
                }
            }

            //If we don't have a story this must be a new game sot give us a story
            if (mStory == "")
                mStory = getNewStory();

            if (mWingameItem == null)
            {
                float amount = WinGameItem.getPriceBasedOnDifficulty(GameDifficulty);
                WinGameItem item = new WinGameItem(amount, amount);
                mBoat.Inventory.addItem(item);
                mWingameItem = item;
            }
        }

        public Model()
        {
            //Create a paused model by default
            pause();
        }

        ///// <summary>
        ///// Initializes a new instance of the <see cref="Model"/> class.
        ///// </summary>
        //public Model()
        //    : base(337, 400, 100, 100)
        //{
        //    mSoundManager = new SoundManager();
        //    mOldKeyboardState = Keyboard.GetState();
        //    mWorldObjects = new List<SceneObject>();
        //    mProjectiles = new List<SceneObject>();
        //    mEquippableObjects = new List<UWB_SceneNode>();

        //    mWaterBounds = new UWB_BoundingBox();
        //    mWaterBounds.setCorners(new Vector3(mWorldBounds.getMin().X, mWorldBounds.getMin().Y, mWorldBounds.getMin().Z),
        //                            new Vector3(mWorldBounds.getMax().X, mWorldBounds.getMax().Y * .8f, mWorldBounds.getMax().Z));

        //    //Set up the scene
        //    mScene = new UWB_SceneNode("Scene");

        //    //Set up our backdrop
        //    mBackDrop = createBackDrop();

        //    UWB_SceneNode node = new UWB_SceneNode("Backdrop");
        //    node.setPrimitive(mBackDrop);
        //    mScene.insertChildNode(node);

        //    // add the walls that make up our level
        //    List<SceneObject> walls = buildWallsForLevel();

        //    if ((walls != null) && (walls.Count > 0))
        //    {
        //        foreach (Wall wall in walls)
        //        {
        //            addToScene(wall);
        //            mWorldObjects.Add(wall);
        //        }
        //    }

        //    // add a shark
        //    Shark shark = new Shark(new Vector3(0, 100, 0), 60);
        //    //shark.setVelocity(new Vector3(-30, 0, 0));
        //    shark.setVelocity(Vector3.Zero);
        //    addToScene(shark);
        //    mWorldObjects.Add(shark);

        //    // add a boat
        //    mBoat = new Boat(new Vector3(0, mWaterBounds.getMax().Y, 0), 100);
        //    addToScene(mBoat);

        //    //Initialize the stores items
        //    Item item = new Knife(Vector3.Zero, 10);
        //    mBoat.Inventory.addItem(item);
        //    item = new SpearGun(Vector3.Zero, 10, true);
        //    mBoat.Inventory.addItem(item);
        //    item = new AirTank(mOriginalHeroLocation, 5, 12);
        //    mBoat.Inventory.addItem(item);

        //    //add the hero
        //    mHero = new Hero("Hero", mOriginalHeroLocation, 5, 12, 8, 3, 5, 10, 4);
        //    addToScene(mHero);
        //}

        #endregion

        #region Public Methods

        /// <summary>
        /// Rotate the hero's arm to aim at a point.
        /// </summary>
        /// <param name="aimLocation">Location to aim at.</param>
        public void heroAimAt(Vector3 aimLocation)
        {
            if (mHero != null)
            {
                mHero.aimAt(mDrawHelper, aimLocation);
            }
        }

        /// <summary>
        /// Gets the water bounds.
        /// </summary>
        /// <returns></returns>
        public UWB_BoundingBox getWaterBounds()
        {
            return mWaterBounds;
        }

        public Hero getHero()
        {
            return mHero;
        }

        public float getHeroMoney()
        {
            return mHero.Money;
        }

        public float getHeroBagCount()
        {
            return mHero.BagCount;
        }

        public float getHeroBagSize()
        {
            return mHero.BagSize;
        }

        public float getHeroAir()
        {
            return mHero.Air;
        }

        public float getHeroAirMax()
        {
            return mHero.AirMax;
        }

        public float getHeroDiveDepth()
        {
            return mHero.getDiveDepth();
        }

        public float getHeroMaxDiveDepth()
        {
            return mHero.getMaxDiveDepth();
        }

        public TimeSpan getHeroDiveTime()
        {
            return mHero.getDiveTime();
        }

        public TimeSpan getHerDiveTimeLeft()
        {
            return mHero.getDiveTimeLeft();
        }

        /// <summary>
        /// Gets the hero bounds.
        /// </summary>
        /// <returns></returns>
        public UWB_BoundingBox getHeroBounds()
        {
            return mHero.getBounds(mDrawHelper, false);
        }

        /// <summary>
        /// Gets the draw helper.
        /// </summary>
        /// <returns></returns>
        public UWB_XNADrawHelper getDrawHelper()
        {
            return mDrawHelper;
        }

        //TODO: remove before turning in.
        public UWB_SceneNode getShark()
        {
            if ((mWorldObjects != null) && (mWorldObjects.Count > 0))
            {
                return mWorldObjects[0];
            }

            return null;
        }

        /// <summary>
        /// Updates the simulation.
        /// </summary>
        public override void updateSimulation()
        {
            //If we are paused don't update
            if (mClock.isPaused())
                return;

            bool revertedToPreviousHeroPosition = false;

            eLevelofDetail lod = mDrawHelper.getLod();
            TimeSpan timespan = mClock.getTimeElapsed();
            float elapsedSeconds = (float)timespan.TotalSeconds;

            RandomlyGenerateShark();

            //Animate all the world objects
            foreach (SceneObject obj in mWorldObjects)
            {
                obj.animate(elapsedSeconds);
            }

            //Get the hero's bounds to do collision on
            UWB_BoundingBox heroBounds = mHero.getBounds(mDrawHelper, false);

            //get the bounds of just the hero's front (excluding arms and feet)
            UWB_BoundingBox heroFrontBounds = new UWB_BoundingBox();
            UWB_SceneNode node = mHero.findNode("Torso");
            if (node != null)
            {
                UWB_BoundingBox bounds = null;
                mHero.getNodeBounds(node, true, ref bounds, mDrawHelper, false);
                heroFrontBounds.setCorners(bounds.getMin(), bounds.getMax());
            }
            node = mHero.findNode("Head");
            if (node != null)
            {
                UWB_BoundingBox bounds = null;
                mHero.getNodeBounds(node, true, ref bounds, mDrawHelper, false);
                heroFrontBounds.add(bounds);
            }
            

            float totalHeightOfWaterColumn = mWaterBounds.getHeight();

            // Get portion of top where air does not deplete based on water bounds height
            float noAirDepletionHeight = totalHeightOfWaterColumn * .028f;

            if (heroBounds.getMax().Y <= mWaterBounds.getMax().Y - noAirDepletionHeight)
            {
                //Use up some of the hero's air
                float amount = 0.01f;

                // air usage is based on depth - use height of water bounds so the world bounds going to zero don't mess us up.
                Vector3 heroCenter = heroBounds.getCenter();

                // the origin in our world is at worldHeight/2, so we can use that as an offset.
                float offset = mWorldBounds.getHeight() / 2;

                if (mHero.hasAirTank())
                {
                    // get depth and multiply to get depth-based amount of air lost this update.
                    amount += ((totalHeightOfWaterColumn - (heroCenter.Y + offset)) * .002f);
                }
                else
                {
                    amount += elapsedSeconds * 1.5f; // not depth-based with no tank
                }

                // decrease the air.
                mHero.decreaseAir(amount);

                //Update our stats
                mHero.addToDiveTime(timespan);
                mHero.setAirLossRate(amount / elapsedSeconds);
            }
            else
            {
                mHero.resetDiveTime();

                // we need to fill up the air if he has no tank here.
                mHero.resetAir();
            }

            //Check if the hero collided with the boat
            UWB_BoundingBox boatVolume = mBoat.getBounds(mDrawHelper, false);
            Vector3 collisionPosition = Vector3.Zero;
            if (heroFrontBounds.intersects(boatVolume, ref collisionPosition))
            {
                //Open the store menu an pause everything
                if (mBoat.mInStore == false)
                {
                    openStore();

                    //Flag that we entered the store so we don't keep colliding with it
                    mBoat.mInStore = true;
                }
            }
            else if(mBoat.mInStore)
            {
                //Get the biggest dimension of the hero and the position of the hero
                float biggestDistance = Math.Max(heroFrontBounds.getWidth(), heroFrontBounds.getHeight());

                //Check if hero no matter how he moves won't collide with the boat
                if (mHero.Position.Y + biggestDistance < boatVolume.getMin().Y)
                {
                    //Flag that we are outside of the store so can collide with it again
                    mBoat.mInStore = false;
                }
            }

            //Check if our hero hit a collectable
            for (int i = mCollectibles.Count - 1; i >= 0; i--)
            {
                Item item = mCollectibles[i];

                //Get the projectiles bounds 
                UWB_BoundingVolume bounds = item.getBounds(mDrawHelper, false);

                //If outside the world remove the object
                collisionPosition = Vector3.Zero;
                if (bounds != null && bounds.intersects(heroBounds, ref collisionPosition))
                {
                    // only remove collectible if the hero added it to his bag.
                    if (mHero.addCollectible(item))
                    {
                        //Remove the collectable
                        mCollectibles.RemoveAt(i);
                        removeFromScene(item);
                    }
                }
            }

            //Move our world objects and cache their bounds so we don't have to
            //calculate them twice.
            List<UWB_BoundingVolume> worldObjectsBounds = new List<UWB_BoundingVolume>();
            for (int i = 0; i < mWorldObjects.Count; i++)
            {
                SceneObject obj = mWorldObjects[i];

                // no need to move walls, they don't have velocity
                if (!(mWorldObjects[i] is Wall))
                {
                    //Update the world objects position
                    obj.moveNodeByVelocity(elapsedSeconds);
                }

                //cache the bounds
                worldObjectsBounds.Add(obj.getBounds(mDrawHelper, false));
            }

            // Check if our projectiles hit anything
            for (int i = mProjectiles.Count - 1; i >= 0; i--)
            {
                UWB_SceneNode projectile = mProjectiles[i];

                //Update the projectiles position
                projectile.moveNodeByVelocity(elapsedSeconds);

                //Get the projectiles bounds 
                UWB_BoundingVolume projectileBounds = projectile.getBounds(mDrawHelper, false);

                //If outside the world remove the object
                collisionPosition = Vector3.Zero;
                if (projectileBounds != null && projectileBounds.isOutside(mWorldBounds))
                {
                    mProjectiles.RemoveAt(i);
                    removeFromScene(projectile);
                    continue;
                }

                //Check if we hit a world object
                for (int j = mWorldObjects.Count - 1; j >= 0; j--)
                {
                    SceneObject obj = mWorldObjects[j];

                    //If there was a collision with a world object remove it (unless it's a wall) and remove our projectile
                    collisionPosition = Vector3.Zero;
                    UWB_BoundingVolume bounds = worldObjectsBounds[j];
                    if (bounds != null && bounds.intersects(projectileBounds, ref collisionPosition))
                    {
                        if (!(obj is Wall))
                        {
                            //Remove the world object
                            mWorldObjects.RemoveAt(j);
                            worldObjectsBounds.RemoveAt(j);
                            removeFromScene(obj);
                        }

                        //Remove the projectile
                        mProjectiles.RemoveAt(i);
                        removeFromScene(projectile);

                        //Since the sphere is gone no need to check other world objects
                        break;
                    }
                }
            }

            //Now check if our world objects are outside the world or collided with our hero
            mHero.resetMoves();
            for (int i = mWorldObjects.Count - 1; i >= 0; i--)
            {
                SceneObject obj = mWorldObjects[i];
                UWB_BoundingVolume bounds = worldObjectsBounds[i];

                //Check if we went outside the world (if it's not a wall)
                collisionPosition = Vector3.Zero;

                if (!(obj is Wall))
                {
                    if (bounds != null && bounds.isOutside(mWorldBounds))
                    {
                        mWorldObjects.RemoveAt(i);
                        worldObjectsBounds.RemoveAt(i);
                        removeFromScene(obj);
                        continue;
                    }
                }

                if (mHero.IsAlive && bounds != null) // only collide hero if he is alive and we have a bounds
                {
					UWB_BoundingBox knifeBounds = mHero.GetKnifeBounds();

                    if (knifeBounds != null)
                    {
                        // check if any world object hit the knife and act accordingly
                        collisionPosition = Vector3.Zero;
                        if (bounds.intersects(knifeBounds, ref collisionPosition))
                        {
                            if (!(obj is Wall))
                            {
                                //Remove the world object
                                mWorldObjects.RemoveAt(i);
                                worldObjectsBounds.RemoveAt(i);
                                removeFromScene(obj);
								continue;
                            }
                        }
                    }

                    // Check if we hit a wall (use only front bounds for walls)
                    collisionPosition = Vector3.Zero;
                    if (obj is Wall && heroFrontBounds != null)
                    {
                        if (bounds.intersects(heroFrontBounds, ref collisionPosition))
                        {
                            // colliding with wall, so we need to revert back to hero's previous position
                            UWB_XFormInfo tempHeroXForm = mHero.getXFormInfo();
                            tempHeroXForm.setTranslation(mPreviousHeroTranslation);
                            tempHeroXForm.setRotationDegrees(mPreviousHeroRotationDegrees);
                            mHero.setXFormInfo(tempHeroXForm);

                            // also, set velocity to zero.  He should stop moving, preferably with a thud noise.
                            mHero.setVelocity(Vector3.Zero);

                            revertedToPreviousHeroPosition = true; // important to set this so it skips the updated position this time...


                            // do whatever the hero is going to do as a result of the collision
                            //mHero.CollisionResponse(mWorldObjects[i], collisionPosition);
                        }
                    }
                    else if(bounds.intersects(heroBounds, ref collisionPosition))
                    {
                        // do whatever the hero is going to do as a result of the collision
                        mHero.CollisionResponse(mWorldObjects[i], collisionPosition);

                        // do whatever the other object is going to do as a result of the collision
                        mWorldObjects[i].CollisionResponse(mHero, collisionPosition);
                    }
					else
                    {
                        // there was no collision, but if the hero saw a shark, we want to do the saw shark animation (he can see for 70 units)
                        if (mWorldObjects[i] is Shark)
                        {
                            float distFromShark = Vector3.Distance(heroBounds.getCenter(), bounds.getCenter());

                            if (distFromShark < 70)
                            {
                                mHero.SawShark();
                            }
                        }
                    }
                }
            }

            if (!revertedToPreviousHeroPosition)
            {
                // Remember this position for next time, because we didn't collide this time, so we might
                // have to revert here later if we collide on the next update.
                this.mPreviousHeroTranslation = mHero.getXFormInfo().getTranslation();
                this.mPreviousHeroRotationDegrees = mHero.getXFormInfo().getRotationInDegrees();

                //Move the hero by velocity
                mHero.moveNodeByVelocity(elapsedSeconds);
            }

            if (mHero.isMoving())
                setZoomedViewCenter(mHero.Position);

            //Animate the hero
            mHero.animate(elapsedSeconds);

            base.updateSimulation();

            // check if we need to restart the game - hero is dead and done dying.
            if (!mHero.IsAlive && mHero.IsDoneDying())
            {
                Program.showNewGameDialog();
            }

            //Check if we won the game
            if (mHero.IsAlive && mHero.hasWon())
            {
                Program.showWonDialog();
            }
        }

        /// <summary>
        /// Open the store window, populate it and pause the game
        /// </summary>
        public void openStore()
        {
            StoreWindow store = new StoreWindow();
            store.HelpRequested += new System.Windows.Forms.HelpEventHandler(store_HelpRequested);
            store.StoreCloseCallback += new StoreWindow.StoreCloseEventHanlder(onStoreClose);
            store.ini(mBoat.Inventory, mHero);
            store.Show();

            Program.pause();
        }

        public void store_HelpRequested(object obj, System.Windows.Forms.HelpEventArgs e)
        {
            Program.openHelpMenu("Boat");
            e.Handled = true;
        }

        public void onStoreClose()
        {
            Program.play();
        }

        /// <summary>
        /// Draws the model.
        /// </summary>
        public override void drawModel()
        {
            eLevelofDetail lod = mDrawHelper.getLod();

            mDrawHelper.initializeModelTransform();
            mDrawHelper.pushModelTransform();

            // draw scene
            mScene.draw(lod, mDrawHelper);

            // pop our custom settings off
            mDrawHelper.popModelTransform();
        }

        /// <summary>
        /// Updates the input.
        /// </summary>
        public void updateInput()
        {
            KeyboardState newKeyboardState = Keyboard.GetState();

            //Update the hero's velocity based on the arrow keys
			if (mHero.IsAlive)
			{
	            float speed = 25.0f;
                if (newKeyboardState.IsKeyDown(Keys.Right) || newKeyboardState.IsKeyDown(Keys.D))
                    mHero.moveRight(speed);
                else if (newKeyboardState.IsKeyDown(Keys.Left) || newKeyboardState.IsKeyDown(Keys.A))
                    mHero.moveLeft(speed);

                if (newKeyboardState.IsKeyDown(Keys.Up) || newKeyboardState.IsKeyDown(Keys.W))
                    mHero.moveUp(speed);
                else if (newKeyboardState.IsKeyDown(Keys.Down) || newKeyboardState.IsKeyDown(Keys.S))
                    mHero.moveDown(speed);

                //Allow the user to pause the game
                if (newKeyboardState.IsKeyDown(Keys.P))
                {
                    Program.onUserPausedGame();
                }
            }

            // Update saved state.
            mOldKeyboardState = newKeyboardState;
        }

        #endregion

        #region Event Service Methods



        #endregion

        #region Private Methods

        /// <summary>
        /// Use whatever is equipped to the hero
        /// </summary>
        public void useEquppedItem()
        {
            Hero hero = getHero();

            if (hero != null)
            {
                SceneObject projectile = hero.useEquippedItem();
                if (projectile != null)
                {
                    mProjectiles.Add(projectile);
                    addToScene(projectile);
                }
            }
        }

        /// <summary>
        /// Updates the primitive location for velocity inside another volume.
        /// </summary>
        /// <param name="primitive">The primitive.</param>
        /// <param name="containingVolume">The containing volume.</param>
        private void updatePrimitiveLocationForVelocityInsideAnotherVolume(UWB_Primitive primitive, UWB_BoundingVolume containingVolume)
        {
            // doing it this way ensures it never gets "stuck" where it is just barely over the line, but the reversed
            // velocity is not enough to bring it back completely inside the outer bounds, so it reverses again, endlessly...
            if (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax().X >= containingVolume.getMax().X)
            {
                // switch to negative velocity
                Vector3 vel = primitive.getVelocity();
                if (vel.X > 0)
                {
                    vel.X = vel.X * -1;
                }
                primitive.setVelocity(vel);
            }
            else if (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin().X <= containingVolume.getMin().X)
            {
                // switch to positive velocity
                Vector3 vel = primitive.getVelocity();
                if (vel.X < 0)
                {
                    vel.X = vel.X * -1;
                }
                primitive.setVelocity(vel);
            }

            if (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax().Y >= containingVolume.getMax().Y)
            {
                // switch to negative velocity
                Vector3 vel = primitive.getVelocity();
                if (vel.Y > 0)
                {
                    vel.Y = vel.Y * -1;
                }
                primitive.setVelocity(vel);
            }
            else if (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin().Y <= containingVolume.getMin().Y)
            {
                // switch to positive velocity
                Vector3 vel = primitive.getVelocity();
                if (vel.Y < 0)
                {
                    vel.Y = vel.Y * -1;
                }
                primitive.setVelocity(vel);
            }
        }

        /// <summary>
        /// Updates the primitive by velocity.
        /// </summary>
        /// <param name="primitive">The primitive.</param>
        /// <param name="elapsedTime">The elapsed time.</param>
        private void updatePrimitiveByVelocity(UWB_Primitive primitive, UWB_BoundingVolume containingVolume, float elapsedSeconds)
        {
            Vector3 updateVector = primitive.getVelocity() * elapsedSeconds;
            Vector3 newCenter = primitive.getLocation() + updateVector;

            // first, check if the inner object is narrower than the bounds in the X direction.  Only correct X if so. (check the same for Y later).
            float primWidth = primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax().X - primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin().X;
            float containingVolumeWidth = containingVolume.getMax().X - containingVolume.getMin().X;
            if (primWidth < containingVolumeWidth)
            {
                // check if we went over our bounds or not and correct
                if (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin().X < containingVolume.getMin().X)
                {
                    // move the center X over to the right by the difference, which should get it back in line.
                    newCenter.X += (containingVolume.getMin().X - primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin().X);
                }
                else if (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax().X > containingVolume.getMax().X)
                {
                    // move the center X over to the left by the difference, which should get it back in line.
                    newCenter.X -= (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax().X - containingVolume.getMax().X);
                }
            }

            float primHeight = primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax().Y - primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin().Y;
            float containingVolumeHeight = containingVolume.getMax().Y - containingVolume.getMin().Y;
            if (primHeight < containingVolumeHeight)
            {
                // check if we went over our bounds or not and correct
                if (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin().Y < containingVolume.getMin().Y)
                {
                    // move the center X up by the difference, which should get it back in line.
                    newCenter.Y += (containingVolume.getMin().Y - primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin().Y);
                }
                else if (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax().Y > containingVolume.getMax().Y)
                {
                    // move the center X down by the difference, which should get it back in line.
                    newCenter.Y -= (primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax().Y - containingVolume.getMax().Y);
                }
            }

            primitive.moveTo(newCenter.X, newCenter.Y);
        }

        private void addToScene(UWB_SceneNode node)
        {
            mScene.insertChildNode(node);
            Program.updateSceneDropDown();

            N_Utility.echoToStatusArea("Projectiles: " + mProjectiles.Count + "    World Objects: " + mWorldObjects.Count + "    Scene Objects: " + mScene.numChildren());
        }

        private void removeFromScene(UWB_SceneNode node)
        {
            mScene.removeChildNode(node);
            Program.updateSceneDropDown();

            N_Utility.echoToStatusArea("Projectiles: " + mProjectiles.Count + "    World Objects: " + mWorldObjects.Count + "    Scene Objects: " + mScene.numChildren());
        }

        /// <summary>
        /// Returns the number of sharks in world.
        /// </summary>
        /// <returns>Returns the number of sharks in world.</returns>
        private int numSharksInWorld()
        {
            int output = 0;

            if ((mWorldObjects != null) && (mWorldObjects.Count > 0))
            {
                foreach (SceneObject obj in mWorldObjects)
                {
                    if (obj is Shark)
                    {
                        output++;
                    }
                }
            }

            return output;
        }

        /// <summary>
        /// Randomlies the generate shark.
        /// </summary>
        /// <param name="rand">The rand.</param>
        private void RandomlyGenerateShark()
        {
            // pick some random time based on the clock to generate sharks
            if (((this.mGameDifficulty == eGameDifficulty.Easy) && (DateTime.Now.Ticks % 37 == 0) && (numSharksInWorld() < 3)) ||
                ((this.mGameDifficulty == eGameDifficulty.Hard) && (DateTime.Now.Ticks % 13 == 0) && (numSharksInWorld() < 8)))
            {
                Vector3 location = Vector3.Zero;
                Vector3 velocity = Vector3.Zero;
                Vector3 waterBoundsMin = mWaterBounds.getMin();
                Vector3 waterBoundsMax = mWaterBounds.getMax();
                waterBoundsMax.Y = waterBoundsMax.Y - 10; // don't want any sharks appearing as if they are in the air...

                // set the height of the window where the shark will enter randomly
                location.Y = N_Utility.GenerateRandomFloat(waterBoundsMin.Y, waterBoundsMax.Y);

                // set the shark length randomly...could have some real monsters!
                float length = N_Utility.GenerateRandomFloat(20, 150);

                // set velocity factor based on length
                float xVelocity = length * .5f;

                // pick which side the shark will enter from at random
                if (N_Utility.GenerateRandomFloat(0, 10) < 5)
                {
                    // enter from left and swim right
                    velocity.X = xVelocity;
                    location.X = waterBoundsMin.X + 1f; // we want the sharks to come in from the side naturally, not just appear out of thin air...but they must be somewhat within the world bounds or they will be removed.
                }
                else
                {
                    // enter from right and swim left
                    velocity.X = -xVelocity;
                    location.X = waterBoundsMax.X - 1f; // we want the sharks to come in from the side naturally, not just appear out of thin air...but they must be somewhat within the world bounds or they will be removed.
                }

                // add a shark
                Shark shark = new Shark(location, length);
                shark.setVelocity(velocity);

                // make sure that we don't place a new shark right on top of the hero - that sucks.
                UWB_BoundingBox heroBounds = mHero.getBounds(mDrawHelper, false);
                UWB_BoundingBox sharkBounds = shark.getBounds(mDrawHelper, false);

                if (sharkBounds != null)
                {
                    Vector3 collisionPosition = Vector3.Zero;

                    // Make sure the shark is at least 70 away from the hero in easy mode - as far as he can see...
                    float distance = Vector3.Distance(heroBounds.getCenter(), sharkBounds.getCenter());

                    // Also, only add shark if he does not intersect with the hero.
                    if ((!sharkBounds.intersects(heroBounds, ref collisionPosition)) && 
                        ((mGameDifficulty == eGameDifficulty.Easy && distance > 70) || (mGameDifficulty == eGameDifficulty.Hard)))
                    {
                        addToScene(shark);
                        mWorldObjects.Add(shark);
                    }
                }
            }
        }

        /// <summary>
        /// Creates the back drop.
        /// </summary>
        /// <returns></returns>
        private UWB_PrimitiveList createBackDrop()
        {
            mBackDrop = new UWB_PrimitiveList();
            float width = mWorldBounds.getWidth();
            float height = mWorldBounds.getHeight();
            float sectionHeight = (float)Math.Round(height / 3.0f);
            UWB_PrimitiveRectangle fill = new UWB_PrimitiveRectangle();
            fill.setCorners(mWorldBounds.getMin(), mWorldBounds.getMax());
            fill.enableTexturing(true);
            fill.setTextureFileName("BackgroundMiddle");
            fill.useTextureAspectRatio(true);
            mBackDrop.append(fill);

            Vector3 max = mWorldBounds.getMin();
            Vector3 min = mWorldBounds.getMin();
            max.Y += sectionHeight;
            max.X += width;
            max.Y += (float)Math.Round(height * .0125f);
            min.Y += (float)Math.Round(height * .0125f);
            UWB_PrimitiveRectangle bottom = new UWB_PrimitiveRectangle();
            bottom.setCorners(min, max);
            bottom.enableTexturing(true);
            bottom.setTextureFileName("BackgroundBottom");
            bottom.useTextureAspectRatio(true);
            bottom.setTextureScale(2.50f);
            bottom.enableBlending(true);
            mBackDrop.append(bottom);

            min.X = mWorldBounds.getMin().X;
            min.Y = mWorldBounds.getMax().Y;
            max = min;
            min.Y -= sectionHeight;
            max.X += width;
            max.Y += (float)Math.Round(height * .0625f);
            min.Y += (float)Math.Round(height * .0625f);
            UWB_PrimitiveRectangle top = new UWB_PrimitiveRectangle();
            top.setCorners(min, max);
            top.enableTexturing(true);
            top.setTextureFileName("BackgroundTop");
            top.useTextureAspectRatio(true);
            top.setTextureScale(2.50f);
            top.enableBlending(true);
            mBackDrop.append(top);

            return mBackDrop;
        }

        public void tempAddToLevel()
        {
            //Vector3 worldMin = mWorldBounds.getMin();
            //Vector3 worldMax = mWorldBounds.getMax();

            //float worldMinX = worldMin.X + .01f; // all the +/- .01f is to control for intersecting with the outside edges and being removed.
            //float worldMinY = worldMin.Y + .01f;
            //float worldMaxX = worldMax.X - .01f;
            //float worldMaxY = worldMax.Y - .01f;
            //float worldWidth = worldMaxX - worldMinX;
            //float worldHeight = worldMaxY - worldMinY;

            //Coin coin = new Coin(new Vector3(worldMinX + (worldWidth * .05f), worldMinY + 20, 0), 2, "RawGold", eCollectibleCondition.Mint, 1);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .17f), worldMinY + 20, 0), 2, "RawGold", 35, 45);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .25f), worldMinY + 20, 0), 2, "RawGold", 35, 45);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .37f), worldMinY + 20, 0), 2, "RawGold", 35, 45);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .47f), worldMinY + 20, 0), 2, "RawGold", 45, 55);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .55f), worldMinY + 20, 0), 2, "RawGold", 45, 65);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .65f), worldMinY + 20, 0), 2, "RawGold", 45, 75);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .75f), worldMinY + 20, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .85f), worldMinY + 20, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .97f), worldMinY + 30, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .88f), worldMinY + 46, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .78f), worldMinY + 66, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMinX + (worldWidth * .68f), worldMinY + 86, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .14f), worldMinY + (worldHeight * .3f) + 20, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .09f), worldMinY + (worldHeight * .3f) + 20, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .04f), worldMinY + (worldHeight * .3f) + 20, 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .195f), worldMinY + (worldHeight * .5f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .180f), worldMinY + (worldHeight * .51f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .165f), worldMinY + (worldHeight * .5f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .150f), worldMinY + (worldHeight * .51f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .135f), worldMinY + (worldHeight * .5f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .120f), worldMinY + (worldHeight * .51f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .105f), worldMinY + (worldHeight * .5f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .090f), worldMinY + (worldHeight * .51f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .075f), worldMinY + (worldHeight * .5f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .060f), worldMinY + (worldHeight * .51f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .045f), worldMinY + (worldHeight * .5f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .030f), worldMinY + (worldHeight * .51f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);

            //coin = new Coin(new Vector3(worldMaxX - (worldWidth * .015f), worldMinY + (worldHeight * .5f), 0), 2, "RawGold", 45, 85);
            //mCollectibles.Add(coin);
            //addToScene(coin);
        }

        private List<SceneObject> buildWallsForLevel()
        {
            List<SceneObject> output = new List<SceneObject>();

            UWB_BoundingBox worldBounds = getWorldBounds();

            Vector3 worldMin = worldBounds.getMin();
            Vector3 worldMax = worldBounds.getMax();

            float worldMinX = worldMin.X + .01f; // all the +/- .01f is to control for intersecting with the outside edges and being removed.
            float worldMinY = worldMin.Y + .01f;
            float worldMaxX = worldMax.X - .01f;
            float worldMaxY = worldMax.Y - .01f;
            float worldWidth = worldMaxX - worldMinX;
            float worldHeight = worldMaxY - worldMinY;

            Color oceanFloorColor = new Color(244, 164, 96);
            Color rustyIBeamColor = new Color(240, 92, 6);
            Color woodColor = new Color(139, 69, 19);
            Color coralColor = Color.Coral;


            // ocean floor
            Wall wall01 = new Wall(new Vector3(worldMinX, worldMinY, 0), worldWidth, 5, oceanFloorColor, eWallTextures.Sand, eOrientation.Horizontal); 
            output.Add(wall01);

            // left most post (rusty I-Beam?)
            Wall wall02 = new Wall(new Vector3(worldMinX + (worldWidth * .1f), worldMinY + wall01.getThickness(), 0)
                      , ((worldHeight / 2) * .4f), 10, rustyIBeamColor, eWallTextures.MetalBeam, eOrientation.Vertical);
            output.Add(wall02);

            // 2nd from left post (rusty I-Beam?)
            Wall wall03 = new Wall(new Vector3(worldMinX + (worldWidth * .3f), worldMinY + wall01.getThickness(), 0)
                      , ((worldHeight / 2) * .85f), 10, rustyIBeamColor, eWallTextures.MetalBeam, eOrientation.Vertical);
            output.Add(wall03);

            // boat hull bottom
            Wall wall04 = new Wall(new Vector3(worldMaxX - (worldWidth * .1f), worldMinY + wall01.getThickness(), 0)
                      , (worldWidth * .1f), 20, woodColor, eWallTextures.Wood, eOrientation.Horizontal);
            output.Add(wall04);

            // boat hull 2nd chunk from bottom
            Wall wall05 = new Wall(new Vector3(worldMaxX - (worldWidth * .2f), wall04.getMin().Y + wall04.getThickness(), 0)
                      , (worldWidth * .13f), 20, woodColor, eWallTextures.Wood, eOrientation.Horizontal);
            output.Add(wall05);

            // boat hull 3rd chunk from bottom
            Wall wall06 = new Wall(new Vector3(worldMaxX - (worldWidth * .3f), wall05.getMin().Y + wall05.getThickness(), 0)
                      , (worldWidth * .13f), 20, woodColor, eWallTextures.Wood, eOrientation.Horizontal);
            output.Add(wall06);

            // boat hull 4th chunk from bottom
            Wall wall07 = new Wall(new Vector3(worldMaxX - (worldWidth * .4f), wall06.getMin().Y + wall06.getThickness(), 0)
                      , (worldWidth * .13f), 20, woodColor, eWallTextures.Wood, eOrientation.Horizontal);
            output.Add(wall07);

            // random shelf in middle of boat
            Wall wall08 = new Wall(new Vector3(worldMaxX - (worldWidth * .18f), worldMinY + (worldHeight * .3f), 0)
                      , (worldWidth * .18f), 15, woodColor, eWallTextures.Wood, eOrientation.Horizontal);
            output.Add(wall08);

            // shelf treasure chest sits on
            Wall wall09 = new Wall(new Vector3(worldMaxX - (worldWidth * .25f), worldMinY + (worldHeight * .45f), 0)
                      , (worldWidth * .25f), 15, woodColor, eWallTextures.Wood, eOrientation.Horizontal);
            output.Add(wall09);

            // lip on treasure chest shelf
            Wall wall10 = new Wall(new Vector3(wall09.getMin().X, wall09.getMin().Y + wall09.getThickness(), 0)
                      , (worldHeight * .06f), 15, woodColor, eWallTextures.Wood, eOrientation.Vertical);
            output.Add(wall10);

            // side wall of boat hull
            Wall wall11 = new Wall(new Vector3(worldMaxX - (worldWidth * .4f), worldMinY + (worldHeight * .3f), 0)
                      , (worldHeight * .34f), 15, woodColor, eWallTextures.Wood, eOrientation.Vertical);
            output.Add(wall11);

            // boat main deck on top
            Wall wall12 = new Wall(new Vector3(wall11.getMin().X, wall11.getMin().Y + wall11.length(), 0)
                      , (worldMaxX - wall11.getMin().X), 15, woodColor, eWallTextures.Wood, eOrientation.Horizontal);
            output.Add(wall12);

            // boat top deck cabin side wall
            Wall wall13 = new Wall(new Vector3(wall11.getMin().X + (wall11.getThickness() * 3), wall12.getMin().Y + wall12.getThickness(), 0)
                      , (worldHeight * .1f), 15, woodColor, eWallTextures.Wood, eOrientation.Vertical);
            output.Add(wall13);

            // left side coral wall
            Wall wall14 = new Wall(new Vector3(worldMinX, worldMaxY - (worldHeight * .5f), 0)
                      , (worldHeight * .3f), 15, coralColor, eWallTextures.Rock, eOrientation.Vertical);
            output.Add(wall14);

            return output;
        }

        #endregion

        #region Public Properties for XML Serialization

        [XmlElement("Story")]
        public string StoryXML
        {
            get
            {
                return mStory;
            }
            set
            {
                mStory = value;
            }
        }

        [XmlArray("WorldObjects")]
        [XmlArrayItem("WorldObject")]
        public List<SceneObject> WorldObjectsXML
        {
            get
            {
                return this.mWorldObjects;
            }
            set
            {
                this.mWorldObjects = value;
            }
        }

        [XmlElement("Boat")]
        public Boat BoatXML
        {
            get
            {
                return this.mBoat;
            }
            set
            {
                this.mBoat = value;
            }
        }

        [XmlElement("Hero")]
        public Hero HeroXML
        {
            get
            {
                return this.mHero;
            }
            set
            {
                this.mHero = value;
            }
        }

        [XmlArray("Projectiles")]
        [XmlArrayItem("Projectile")]
        public List<SceneObject> ProjectilesXML
        {
            get
            {
                return this.mProjectiles;
            }
            set
            {
                this.mProjectiles = value;
            }
        }

        [XmlArray("Collectibles")]
        [XmlArrayItem("Collectible")]
        public List<Item> CollectiblesXML
        {
            get
            {
                return mCollectibles;
            }
            set
            {
                mCollectibles = value;
            }
        }

        #endregion
    }
}

