﻿using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using NathanScott_Lib;
using System;
using System.Xml.Serialization;

namespace FinalProject
{
    [Serializable]
    public class AirTank : Item
    {
        private float mAir = 100.0f;
        private float mAirMax = 100.0f;
        private Vector3 pivot;
        private float torsoHeight;
        private float torsoWidth;

        public AirTank()
            : base()
        {

        }

        public AirTank(Vector3 at, float torsoHeight, float torsoWidth):base("AirTank", Properties.Resources.AirTank, 50.00f, 50.00f)
        {
            mRecoverImage = Properties.Resources.AirTank;

            this.pivot = at;
            this.torsoHeight = torsoHeight;
            this.torsoWidth = torsoWidth;
            UWB_PrimitiveList pList = new UWB_PrimitiveList();

            Color airTankColor = new Color(245, 238, 31);
            Color wetSuitColor = new Color(0, 0, 0);

            // some calculations for the air tank
            float airTankBottomY = at.Y + (torsoHeight / 2) + (torsoHeight / 8);
            float airTankTopY = at.Y + (torsoHeight / 2) + (torsoHeight * .75f);
            Vector3 airTankBottomLeftCorner = new Vector3(at.X - (torsoWidth / 2), airTankBottomY, 0);
            Vector3 airTankUpperRightCorner = new Vector3(at.X + (torsoWidth / 6), airTankTopY, 0);
            float airTankYDelta = airTankUpperRightCorner.Y - airTankBottomLeftCorner.Y;
            Vector3 airTankTopCenter = new Vector3(airTankUpperRightCorner.X, at.Y + (torsoHeight / 2) + (torsoHeight / 8) + (airTankYDelta / 2f), 0);

            UWB_PrimitiveRectangle airTank = new UWB_PrimitiveRectangle();
            airTank.setCorners(airTankBottomLeftCorner, airTankUpperRightCorner);
            airTank.setFlatColor(airTankColor);
            airTank.setShadeMode(eShadeMode.smFlat);
            pList.append(airTank);

            UWB_PrimitiveCircle airTankTop = new UWB_PrimitiveCircle();
            airTankTop.setCenter(airTankTopCenter);
            airTankTop.setRadius(airTankYDelta / 2f);
            airTankTop.setFlatColor(airTankColor);
            airTankTop.setShadeMode(eShadeMode.smFlat);
            pList.append(airTankTop);

            UWB_PrimitiveRectangle airTankStrap = new UWB_PrimitiveRectangle();
            airTankStrap.setCorners(new Vector3(airTankTop.getLocation().X - (airTankTop.getRadius() * 3), at.Y + (torsoHeight / 2), 0),
                                    new Vector3(airTankTop.getLocation().X - (airTankTop.getRadius() * 2), airTankTopY, 0));
            airTankStrap.setFlatColor(wetSuitColor);
            airTankStrap.setShadeMode(eShadeMode.smFlat);
            pList.append(airTankStrap);

            setPrimitive(pList);
            mXFormInfo.setPivot(at);
        }

        public float Air
        {
            get
            {
                return mAir;
            }
            set
            {
                mAir = value;
            }
        }

        /// <summary>
        /// Decrease air by an amount
        /// </summary>
        /// <param name="amount">amount to decrees air by</param>
        /// <returns>true if out of air, false if not</returns>
        public bool decreaseAir(float amount)
        {
            float value = mAir - amount;

            mAir = N_Math.Clamp(value, 0, mAirMax);

            return mAir <= 0.0f;
        }

        public float AirMax
        {
            get
            {
                return mAirMax;
            }
            set
            {
                mAirMax = value;
            }
        }

        public override bool isRecoveryable()
        {
            return mAir != mAirMax;
        }

        public override bool performRecoverAction(object obj)
        {
            //Just in case
            if (mAir == mAirMax)
                return true;

            if(obj != null && obj is Hero)
            {
                Hero hero = obj as Hero;

                if (hero.subtractMoney(getRecoverCost()))
                {
                    mAir = mAirMax;
                    return true;
                }
            }

            return false;
        }

        public override float getRecoverCost()
        {
            //May want to make this dependent on air max or how much you have left
            return 5.00f;
        }

        public override bool isEquippable()
        {
            return false;
        }

        #region Public Properties for XML Serialization

        [XmlElement("Pivot")]
        public Vector3 PivotXML
        {
            get
            {
                return this.pivot;
            }
            set
            {
                this.pivot = value;
            }
        }

        [XmlElement("TorsoHeight")]
        public float TorsoHeightXML
        {
            get
            {
                return this.torsoHeight;
            }
            set
            {
                this.torsoHeight = value;
            }
        }

        [XmlElement("TorsoWidth")]
        public float TorsoWidthXML
        {
            get
            {
                return this.torsoWidth;
            }
            set
            {
                this.torsoWidth = value;
            }
        }

        #endregion
    }
}
