﻿using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using System;
using System.Xml.Serialization;

namespace FinalProject
{
    [Serializable]
    public class Knife : Item
    {
        private Vector3 pivot;
        private float length;

        public Knife()
            : base()
        {

        }

        public Knife(Vector3 at, float knifeLength)
            : base("Knife", Properties.Resources.Knife, 30.00f, 30.00f)
        {
            this.pivot = at;
            this.length = knifeLength;
            UWB_PrimitiveList pList = new UWB_PrimitiveList();

            Color handleColor = Color.Black;
            Color bladeColor = Color.Silver;

            Vector3 knifeBaseMin = new Vector3(at.X - (knifeLength * .05f), at.Y - (knifeLength * .3f), 0);
            Vector3 knifeBaseMax = new Vector3(at.X + (knifeLength * .05f), at.Y, 0);
            Vector3 hiltMin = new Vector3(knifeBaseMin.X - (knifeLength * .1f), knifeBaseMin.Y - (knifeLength * .1f), 0);
            Vector3 hiltMax = new Vector3(knifeBaseMax.X + (knifeLength * .1f), knifeBaseMin.Y, 0);
            Vector3 bladeLeft = new Vector3(knifeBaseMin.X - (knifeLength * .05f), hiltMin.Y, 0);
            Vector3 bladeRight = new Vector3(knifeBaseMax.X + (knifeLength * .05f), hiltMin.Y, 0);
            Vector3 bladeTip = new Vector3(at.X, knifeBaseMin.Y - (knifeLength * .6f), 0);

            UWB_PrimitiveRectangle knifeBase = new UWB_PrimitiveRectangle();
            knifeBase.setCorners(knifeBaseMin, knifeBaseMax);
            knifeBase.setFlatColor(handleColor);
            knifeBase.setShadeMode(eShadeMode.smFlat);
            pList.append(knifeBase);

            UWB_PrimitiveRectangle hilt = new UWB_PrimitiveRectangle();
            hilt.setCorners(hiltMin, hiltMax);
            hilt.setFlatColor(handleColor);
            hilt.setShadeMode(eShadeMode.smFlat);
            pList.append(hilt);

            UWB_PrimitiveTriangle blade = new UWB_PrimitiveTriangle();
            blade.setBottomLeftPoint(bladeLeft.X, bladeLeft.Y, bladeLeft.Z);
            blade.setBottomRightPoint(bladeRight.X, bladeRight.Y, bladeRight.Z);
            blade.setTopPoint(bladeTip.X, bladeTip.Y, bladeTip.Z);
            blade.setFlatColor(bladeColor);
            blade.setShadeMode(eShadeMode.smFlat);
            pList.append(blade);

            setPrimitive(pList);
            mXFormInfo.setPivot(at);
        }

        #region Public Properties for XML Serialization

        [XmlElement("Pivot")]
        public Vector3 PivotXML
        {
            get
            {
                return this.pivot;
            }
            set
            {
                this.pivot = value;
            }
        }

        [XmlElement("Length")]
        public float LengthXML
        {
            get
            {
                return this.length;
            }
            set
            {
                this.length = value;
            }
        }

        #endregion
    }
}
