﻿using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using System;
using System.Xml.Serialization;

namespace FinalProject
{
    [Serializable]
    public class SpearGun : Item
    {
        private Vector3 mPivot;
        private float mLength;
        private bool mHasSpear;

        protected int mSpearCount;
        private Spear mSpear = null;

        public SpearGun()
            : base()
        {

        }

        public SpearGun(Vector3 at, float speargunLength, bool hasSpear):base("SpearGun", Properties.Resources.SpearGun, 150.00f, 150.00f)
        {
            mRecoverImage = Properties.Resources.Spear;

            mSpearCount = 10;

            this.mPivot = at;
            this.mLength = speargunLength;
            this.mHasSpear = hasSpear;

            UWB_PrimitiveList pList = new UWB_PrimitiveList();

            Color handleColor = Color.Black;
            Color trimColor = Color.Silver;

            Vector3 handleMin = new Vector3(at.X - (speargunLength * .15f), at.Y - (speargunLength * .1f), 0);
            Vector3 handleMax = new Vector3(at.X + (speargunLength * .15f), at.Y, 0);
            Vector3 midPartMin = new Vector3(handleMax.X, handleMin.Y - (speargunLength * .4f), 0);
            Vector3 midPartMax = new Vector3(handleMax.X + (speargunLength * .2f), handleMax.Y, 0);
            float distanceBetweenMidMinAndMaxX = midPartMax.X - midPartMin.X;
            float offsetToUse = distanceBetweenMidMinAndMaxX * .25f;
            Vector3 barrelMin = new Vector3(midPartMin.X + offsetToUse, midPartMin.Y - (speargunLength * .6f), 0);
            Vector3 barrelMax = new Vector3(midPartMax.X - offsetToUse, midPartMin.Y, 0);

            // we want to draw the spear first, so it looks like the spear is INSIDE the gun.
            if (hasSpear)
            {
                createAttachedSpear();
            }

            UWB_PrimitiveRectangle handle = new UWB_PrimitiveRectangle();
            handle.setCorners(handleMin, handleMax);
            handle.setFlatColor(handleColor);
            handle.setShadeMode(eShadeMode.smFlat);
            pList.append(handle);

            UWB_PrimitiveRectangle midPart = new UWB_PrimitiveRectangle();
            midPart.setCorners(midPartMin, midPartMax);
            midPart.setFlatColor(trimColor);
            midPart.setShadeMode(eShadeMode.smFlat);
            pList.append(midPart);

            UWB_PrimitiveRectangle barrel = new UWB_PrimitiveRectangle();
            barrel.setCorners(barrelMin, barrelMax);
            barrel.setFlatColor(handleColor);
            barrel.setShadeMode(eShadeMode.smFlat);
            pList.append(barrel);

            setPrimitive(pList);
            mXFormInfo.setPivot(at);
        }

        public int SpearCount
        {
            get
            {
                return mSpearCount;
            }
            set
            {
                mSpearCount = value;
            }
        }

        public override bool canUseItem()
        {
            return (mSpear != null && mSpearCount > 0);
        }

        public override SceneObject useItem(UWB_XFormInfo xFormToAttachedNode)
        {
            Model model = Program.getModel();
            if (canUseItem() && model != null && xFormToAttachedNode != null)
            {
                UWB_DrawHelper helper = model.getDrawHelper();

                if (helper != null)
                {
                    UWB_XFormInfo spearXForm = mSpear.getXFormInfo();
                    float xScale = spearXForm.getScale().X;
                    bool add = xScale > 0 ? true : false;

                    // capture the current pivot of the spear - before transformation - need to transform!
                    Vector3 pivot = spearXForm.getPivot();

                    // now add the spear's rotation to the one leading up to the hand just to be safe
                    AddOrSubtractRotation(xFormToAttachedNode, spearXForm, add);

                    // transform with current xForm
                    helper.pushModelTransform();
                    {
                        xFormToAttachedNode.setupModelStack(helper);

                        helper.transformPoint(ref pivot);
                    }
                    helper.popModelTransform();

                    float spearLength = mSpear.SpearLength;

                    // now I need to reset the translate to zero, because we already have the pivot bounds, which is where we place the new spear.
                    xFormToAttachedNode.setTranslation(Vector3.Zero);

                    // recreate spear as its own world object, and give it a velocity
                    Spear worldSpear = new Spear(pivot, spearLength, xFormToAttachedNode);
                    worldSpear.SetSpearColor(Color.Red);

                    Vector3 normalizedVelocityVectorToUse = worldSpear.getNormalizedVelocityVectorForCurrentXForm(xFormToAttachedNode, helper);
                    Vector3 newVelocity = normalizedVelocityVectorToUse * 50f; // some speed...
                    worldSpear.setVelocity(newVelocity);

                    // play the bloop sound as we fire the spear gun
                    UWB_Manager.mAudioManager.playSound(eSoundFiles.Bloop.ToString(), 1, 0, 0, false);

                    //Remove a spear from our count
                    mSpearCount--;
                    if (mSpearCount <= 0)
                    {
                        removeChildNode(mSpear);
                        mSpear = null;
                    }

                    return worldSpear;
                }
            }

            return null;
        }

        public override bool isRecoveryable()
        {
            //Can always add more spears
            return true;
        }

        public override bool performRecoverAction(object obj)
        {
            if (obj != null && obj is Hero)
            {
                Hero hero = obj as Hero;

                if (hero.subtractMoney(getRecoverCost()))
                {
                    mSpearCount += 5;

                    //If we were out create the attached one
                    if (mSpear == null)
                        createAttachedSpear();

                    return true;
                }
            }

            return false;
        }

        public override string getRecoveryText()
        {
            return getRecoverCost().ToString("$0.00 for 5");
        }

        public override float getRecoverCost()
        {
            return 15.00f;
        }

        public override string getInventoryText()
        {
            return mSpearCount.ToString("x 0");
        }

        private void createAttachedSpear()
        {
            Vector3 handleMin = new Vector3(mPivot.X - (mLength * .15f), mPivot.Y - (mLength * .1f), 0);
            Vector3 handleMax = new Vector3(mPivot.X + (mLength * .15f), mPivot.Y, 0);
            Vector3 midPartMin = new Vector3(handleMax.X, handleMin.Y - (mLength * .4f), 0);
            Vector3 midPartMax = new Vector3(handleMax.X + (mLength * .2f), handleMax.Y, 0);
            float distanceBetweenMidMinAndMaxX = midPartMax.X - midPartMin.X;
            float offsetToUse = distanceBetweenMidMinAndMaxX * .25f;
            Vector3 spearAt = new Vector3(midPartMin.X + (offsetToUse * 2), midPartMin.Y, 0);
            mSpear = new Spear(spearAt, (mLength * .75f), null); // null is for a default XFormInfo.
            insertChildNode(mSpear);
        }

        #region Public Properties for XML Serialization

        [XmlElement("Pivot")]
        public Vector3 PivotXML
        {
            get
            {
                return this.mPivot;
            }
            set
            {
                this.mPivot = value;
            }
        }

        [XmlElement("Length")]
        public float LengthXML
        {
            get
            {
                return this.mLength;
            }
            set
            {
                this.mLength = value;
            }
        }

        [XmlElement("HasSpear")]
        public bool HasSpearXML
        {
            get
            {
                return this.SpearCount > 0;
            }
            set
            {
                this.mHasSpear = value;
            }
        }

        #endregion
    }

    [Serializable]
    public class Spear : SceneObject
    {
        private float mSpearLength = 0;
        private Vector3 pivot;
        private Color spearColor;

        public Spear()
            : base()
        {

        }

        public Spear(Vector3 at, float spearLength, UWB_XFormInfo xFormInfoToUse)
        {
            this.pivot = at;
            mName = "Spear";
            this.mSpearLength = spearLength;

            // use passed in XForm if they gave us one.
            if (xFormInfoToUse != null)
            {
                this.setXFormInfo(xFormInfoToUse);
            }

            UWB_PrimitiveList pList = new UWB_PrimitiveList();

            spearColor = Color.Yellow;

            Vector3 shaftEnd = new Vector3(at.X, at.Y - (spearLength * .85f), 0);
            float widthOfTip = spearLength * .2f;
            Vector3 tipLeft = new Vector3(at.X - (widthOfTip/2), shaftEnd.Y, 0);
            Vector3 tipRight = new Vector3(at.X + (widthOfTip / 2), shaftEnd.Y, 0);
            Vector3 tipTip = new Vector3(at.X, shaftEnd.Y - (spearLength * .15f), 0);

            UWB_PrimitiveLine shaft = new UWB_PrimitiveLine();
            shaft.setStartPoint(at.X, at.Y, at.Z);
            shaft.setEndPoint(shaftEnd.X, shaftEnd.Y, shaftEnd.Z);
            shaft.setFlatColor(spearColor);
            shaft.setShadeMode(eShadeMode.smFlat);
            pList.append(shaft);

            UWB_PrimitiveTriangle tip = new UWB_PrimitiveTriangle();
            tip.setBottomLeftPoint(tipLeft.X, tipLeft.Y, tipLeft.Z);
            tip.setBottomRightPoint(tipRight.X, tipRight.Y, tipRight.Z);
            tip.setTopPoint(tipTip.X, tipTip.Y, tipTip.Z);
            tip.setFlatColor(spearColor);
            tip.setShadeMode(eShadeMode.smFlat);
            pList.append(tip);

            setPrimitive(pList);
            mXFormInfo.setPivot(at);
        }

        /// <summary>
        /// Gets or sets the length of the spear.
        /// </summary>
        /// <value>The length of the spear.</value>
        public float SpearLength
        {
            get
            {
                return this.mSpearLength;
            }
            set
            {
                this.mSpearLength = value;
            }
        }

        public void SetSpearColor(Color color)
        {
            UWB_PrimitiveList pList = (UWB_PrimitiveList)this.getPrimitive();

            if ((pList != null) && (pList.count() > 0))
            {
                for (int i = 0; i < pList.count(); i++)
                {
                    pList.primitiveAt(i).setFlatColor(color);
                }
            }
        }

        public Color SpearColor
        {
            get
            {
                return this.spearColor;
            }
            set
            {
                this.spearColor = value;
            }
        }

        /// <summary>
        /// Gets the normalized velocity vector for current X form.
        /// </summary>
        /// <param name="xFormToUse">The x form to use.</param>
        /// <param name="helper">The helper.</param>
        /// <returns></returns>
        public Vector3 getNormalizedVelocityVectorForCurrentXForm(UWB_XFormInfo xFormToUse, UWB_DrawHelper helper)
        {
            Vector3 output = Vector3.Zero;

            // get start point and end point of line, and transform them
            UWB_PrimitiveList primitives = (UWB_PrimitiveList)this.getPrimitive();

            if ((primitives != null) && (primitives.count() > 0))
            {
                UWB_Primitive temp = primitives.primitiveAt(0);

                if (temp != null)
                {
                    UWB_PrimitiveLine shaft = temp as UWB_PrimitiveLine;

                    if (shaft != null)
                    {
                        Vector3 startPoint = shaft.getStartPoint();
                        Vector3 endPoint = shaft.getEndPoint();

                        helper.pushModelTransform();
                        {
                            xFormToUse.setupModelStack(helper);

                            helper.transformPoint(ref startPoint);
                            helper.transformPoint(ref endPoint);
                        }
                        helper.popModelTransform();
                        
                        // now create velocity vector
                        output = endPoint - startPoint;
                        output.Normalize();
                    }
                }
            }

            return output;
        }
    }
}
