﻿using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using System;

namespace FinalProject
{
    [Serializable]
    public class Head : UWB_SceneNode
    {
        public Head()
            : base()
        {

        }

        public Head(Vector3 at, float headRadius, string name) : base(name)
        {
            UWB_PrimitiveList pList = new UWB_PrimitiveList();

            float eyeRadius = headRadius * .2f;
            float maskThickness = headRadius / 20;
            Color skinColor = new Color(238, 171, 121);
            Color maskColor = new Color(0, 0, 0);

            UWB_PrimitiveCircle head = new UWB_PrimitiveCircle();
            head.setCenter(at);
            head.setRadius(headRadius);
            head.setFlatColor(skinColor);
            head.setShadeMode(eShadeMode.smFlat);
            head.setTextureFileName("human_skin");
            head.setTextureScale(1.50f);
            head.enableTexturing(true);
            pList.append(head);

            UWB_SceneNode eyeNode = new UWB_SceneNode("Eye");
            insertChildNode(eyeNode);

            UWB_PrimitiveList pEyeList = new UWB_PrimitiveList();

            UWB_PrimitiveCircle eye = new UWB_PrimitiveCircle();
            eye.setCenter(new Vector3(at.X + (headRadius * 0.62f), at.Y + (headRadius * .4f), 0));
            eye.setRadius(eyeRadius);
            eye.setFlatColor(new Color(255, 255, 255));
            eye.setShadeMode(eShadeMode.smFlat);
            pEyeList.append(eye);

            UWB_PrimitiveCircle iris = new UWB_PrimitiveCircle();
            iris.setCenter(eye.getLocation());
            iris.setRadius(eyeRadius * .65f);
            iris.setFlatColor(new Color(0, 0, 255));
            iris.setShadeMode(eShadeMode.smFlat);
            pEyeList.append(iris);

            UWB_PrimitiveCircle pupil = new UWB_PrimitiveCircle();
            pupil.setCenter(eye.getLocation());
            pupil.setRadius(iris.getRadius() * .5f);
            pupil.setFlatColor(new Color(0,0,0));
            pupil.setShadeMode(eShadeMode.smFlat);
            pEyeList.append(pupil);

            eyeNode.setPrimitive(pEyeList);
            eyeNode.getXFormInfo().setPivot(eye.getLocation());

            UWB_PrimitiveRectangle nose = new UWB_PrimitiveRectangle();
            nose.setCorners(new Vector3(at.X + headRadius - (headRadius/10), at.Y - (headRadius/10), 0),
                            new Vector3(at.X + headRadius + (headRadius / 5), at.Y + (headRadius / 10), 0));
            nose.setFlatColor(skinColor);
            nose.setTextureFileName("human_skin");
            nose.setTextureScale(1.10f);
            nose.enableTexturing(true);
            nose.setShadeMode(eShadeMode.smFlat);
            pList.append(nose);

            UWB_PrimitiveRectangle maskBottom = new UWB_PrimitiveRectangle();
            maskBottom.setFlatColor(maskColor);
            maskBottom.setShadeMode(eShadeMode.smFlat);
            maskBottom.setCorners(new Vector3(eye.getLocation().X - eyeRadius - (headRadius/10), at.Y - eyeRadius, 0),
                                  new Vector3(nose.getLocation().X + nose.getWidth(), at.Y - eyeRadius + maskThickness, 0));
            pList.append(maskBottom);

            UWB_PrimitiveRectangle maskTop = new UWB_PrimitiveRectangle();
            maskTop.setFlatColor(maskColor);
            maskTop.setShadeMode(eShadeMode.smFlat);
            maskTop.setCorners(new Vector3(eye.getLocation().X - eyeRadius - (headRadius / 10), eye.getLocation().Y + eyeRadius + (eyeRadius * .5f), 0),
                               new Vector3(nose.getLocation().X + nose.getWidth(), eye.getLocation().Y + eyeRadius + (eyeRadius * .5f) - maskThickness, 0));
            pList.append(maskTop);

            UWB_PrimitiveRectangle maskLeftSide = new UWB_PrimitiveRectangle();
            maskLeftSide.setFlatColor(maskColor);
            maskLeftSide.setShadeMode(eShadeMode.smFlat);
            maskLeftSide.setCorners(new Vector3(eye.getLocation().X - eyeRadius - (headRadius / 10), at.Y - eyeRadius, 0),
                                    new Vector3(eye.getLocation().X - eyeRadius - (headRadius / 10) + maskThickness, eye.getLocation().Y + eyeRadius + (eyeRadius * .5f), 0));
            pList.append(maskLeftSide);

            UWB_PrimitiveRectangle maskRightSide = new UWB_PrimitiveRectangle();
            maskRightSide.setFlatColor(maskColor);
            maskRightSide.setShadeMode(eShadeMode.smFlat);
            maskRightSide.setCorners(new Vector3(nose.getLocation().X + nose.getWidth(), at.Y - eyeRadius, 0),
                                    new Vector3(nose.getLocation().X + nose.getWidth() + maskThickness, eye.getLocation().Y + eyeRadius + (eyeRadius * .5f), 0));
            pList.append(maskRightSide);

            UWB_PrimitiveRectangle maskStrap = new UWB_PrimitiveRectangle();
            maskStrap.setFlatColor(maskColor);
            maskStrap.setShadeMode(eShadeMode.smFlat);
            maskStrap.setCorners(new Vector3(at.X - headRadius, at.Y, 0),
                                 new Vector3(eye.getLocation().X - eyeRadius - (headRadius / 10), at.Y + maskThickness, 0));
            pList.append(maskStrap);

            UWB_PrimitiveCircle regulatorOuter = new UWB_PrimitiveCircle();
            regulatorOuter.setCenter(new Vector3(eye.getLocation().X + (eyeRadius * 1.2f), eye.getLocation().Y - headRadius * 1.1f, 0));
            regulatorOuter.setRadius(eyeRadius * 2);
            regulatorOuter.setFlatColor(maskColor);
            regulatorOuter.setShadeMode(eShadeMode.smFlat);
            pList.append(regulatorOuter);

            UWB_PrimitiveCircle regulatorInner = new UWB_PrimitiveCircle();
            regulatorInner.setCenter(regulatorOuter.getLocation());
            regulatorInner.setRadius(regulatorOuter.getRadius() * .5f);
            regulatorInner.setFlatColor(new Color(128, 128, 128));
            regulatorInner.setShadeMode(eShadeMode.smFlat);
            pList.append(regulatorInner);

            setPrimitive(pList);
            mXFormInfo.setPivot(new Vector3(at.X - headRadius, at.Y, 0));

            // insert child nodes here if needed.
        }
    }
}
