﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using UWBGL_XNA_Lib13;
using System.Xml.Serialization;

namespace FinalProject
{
    [Serializable]
    [XmlInclude(typeof(Knife))]
    [XmlInclude(typeof(SpearGun))]
    [XmlInclude(typeof(AirTank))]
    [XmlInclude(typeof(Coin))]
    [XmlInclude(typeof(WinGameItem))]
    public class Item : UWB_SceneNode
    {
        protected Image mImage;
        [XmlIgnore]
        public Image mRecoverImage;
        protected float mMaxPrice;
        protected float mMinPrice;
        protected float mPrice;

        public Item()
            : base()
        {

        }

        public Item(string name, Bitmap previewImage, float minPrice, float maxPrice)
            : base(name)
        {
            mImage = previewImage;
            mMinPrice = minPrice;
            mMaxPrice = maxPrice;
        }

        [XmlIgnore]
        public Image PreviewImage
        {
            get
            {
                return mImage;
            }
        }

        /// <summary>
        /// get the maximum price the item can be
        /// </summary>
        public float MaxPrice
        {
            get
            {
                return mMaxPrice;
            }
            set
            {
                mMaxPrice = value;
            }
        }

        /// <summary>
        /// get the minimum price the item can be
        /// </summary>
        public float MinPrice
        {
            get
            {
                return mMinPrice;
            }
            set
            {
                mMinPrice = value;
            }
        }

        /// <summary>
        /// Get/set the current price of the item
        /// </summary>
        public float Price
        {
            get
            {
                return mPrice;
            }
            set
            {
                mPrice = value;
            }
        }

        public virtual bool isRecoveryable()
        {
            return false;
        }

        public virtual bool performRecoverAction(object obj)
        {
            return true;
        }

        public virtual string getRecoveryText()
        {
            return getRecoverCost().ToString("$0.00");
        }

        public virtual float getRecoverCost()
        {
            return 0.00f;
        }

        public virtual bool isEquippable()
        {
            return true;
        }

        public virtual bool canUseItem()
        {
            return false;
        }

        public virtual SceneObject useItem(UWB_XFormInfo xFormToAttachedNode)
        {
            return null;
        }

        public virtual string getInventoryText()
        {
            return string.Empty;
        }
    }
}
