﻿using Microsoft.Xna.Framework;
using System.Collections.Generic;
using UWBGL_XNA_Lib13;
using System;
using System.Xml.Serialization;

namespace FinalProject
{
    [Serializable]
    public class Boat : SceneObject
    {
        public bool mInStore = false;
        private Inventory mInventory = new Inventory();
        private Vector3 pivot;
        private float length;

        public Boat()
            : base()
        {

        }

        public Boat(Vector3 at, float BoatLength):base("Boat")
        {
            this.pivot = at;
            this.length = BoatLength;
            UWB_PrimitiveList mPrimitiveList = new UWB_PrimitiveList();

            Color BoatColorMain = Color.White;
            Color BoatColorTrim = Color.Red;
            Color BoatColorWindow = Color.Gray;

            Vector3 backBottom = new Vector3(at.X - (BoatLength * .4f), at.Y - (BoatLength * .1f), 0);
            Vector3 frontTop = new Vector3(at.X + (BoatLength * .4f), at.Y + (BoatLength * .1f), 0);
            Vector3 frontBottom = new Vector3(frontTop.X, backBottom.Y, 0);
            Vector3 frontTip = new Vector3(frontTop.X + (BoatLength * .2f), frontTop.Y, 0);
            Vector3 windshieldFront = new Vector3(frontTip.X - (BoatLength * .2f), frontTip.Y, 0);
            Vector3 windshieldBack = new Vector3(windshieldFront.X - (BoatLength * .2f), windshieldFront.Y, 0);
            Vector3 windshieldTop = new Vector3(((windshieldFront.X + windshieldBack.X) / 2), windshieldFront.Y + (frontTop.Y - frontBottom.Y), 0);
            Vector3 cabinBackBottom = new Vector3(windshieldBack.X - (BoatLength * .2f), windshieldBack.Y, 0);

            UWB_PrimitiveRectangle boatMain = new UWB_PrimitiveRectangle();
            boatMain.setCorners(backBottom, frontTop);
            boatMain.setFlatColor(BoatColorMain);
            boatMain.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(boatMain);

            UWB_PrimitiveTriangle boatFrontTip = new UWB_PrimitiveTriangle();
            boatFrontTip.setBottomLeftPoint(windshieldFront.X, windshieldFront.Y, windshieldFront.Z);
            boatFrontTip.setBottomRightPoint(frontTip.X, frontTip.Y, frontTip.Z);
            boatFrontTip.setTopPoint(frontBottom.X, frontBottom.Y, frontBottom.Z);
            boatFrontTip.setFlatColor(BoatColorMain);
            boatFrontTip.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(boatFrontTip);

            UWB_PrimitiveRectangle boatCabin = new UWB_PrimitiveRectangle();
            boatCabin.setCorners(cabinBackBottom, windshieldTop);
            boatCabin.setFlatColor(BoatColorMain);
            boatCabin.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(boatCabin);

            UWB_PrimitiveTriangle windshield = new UWB_PrimitiveTriangle();
            windshield.setBottomLeftPoint(windshieldBack.X, windshieldBack.Y, windshieldBack.Z);
            windshield.setBottomRightPoint(windshieldFront.X, windshieldFront.Y, windshieldFront.Z);
            windshield.setTopPoint(windshieldTop.X, windshieldTop.Y, windshieldTop.Z);
            windshield.setFlatColor(BoatColorTrim);
            windshield.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(windshield);

            UWB_PrimitiveRectangle boatCabinWindow = new UWB_PrimitiveRectangle();
            boatCabinWindow.setCorners(new Vector3(cabinBackBottom.X + (BoatLength * .05f), cabinBackBottom.Y + (BoatLength * .05f), 0),
                                       new Vector3(windshieldBack.X, windshieldTop.Y - (BoatLength * .05f), 0));
            boatCabinWindow.setFlatColor(BoatColorWindow);
            boatCabinWindow.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(boatCabinWindow);

            setPrimitive(mPrimitiveList);
            mXFormInfo.setPivot(at);
        }

        public Inventory Inventory
        {
            get
            {
                return mInventory;
            }
            set
            {
                mInventory = value;
            }
        }

        #region Public Properties for XML Serialization

        [XmlElement("InStore")]
        public bool InStoreXML
        {
            get
            {
                return this.mInStore;
            }
            set
            {
                this.mInStore = value;
            }
        }

        [XmlElement("Pivot")]
        public Vector3 PivotXML
        {
            get
            {
                return this.pivot;
            }
            set
            {
                this.pivot = value;
            }
        }

        [XmlElement("Length")]
        public float LengthXML
        {
            get
            {
                return this.length;
            }
            set
            {
                this.length = value;
            }
        }

        #endregion
    }
}
