﻿using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using System;
using System.Xml.Serialization;

namespace FinalProject
{
    [Serializable]
    public class Shark : SceneObject
    {
        private Vector3 pivot;
        private float length;
        private PectoralFin pectoralFin;

        public Shark()
            : base()
        {

        }

        public Shark(Vector3 at, float sharkLength)
            : base("Shark")
        {
            //this guy is definitely deadly!
            mObjType = eSceneObjectType.Deadly;

            this.pivot = at;
            this.length = sharkLength;
            UWB_PrimitiveList mPrimitiveList = new UWB_PrimitiveList();

            Color sharkColor = new Color(128, 128, 128);
            Color pectoralFinColor = new Color(100, 100, 100);
            Color sharkMouthColor = Color.White;

            Vector3 front = new Vector3(at.X + (sharkLength * .45f), at.Y, 0);
            Vector3 back = new Vector3(at.X - (sharkLength * .45f), at.Y, 0);
            Vector3 top = new Vector3(at.X, at.Y + (sharkLength * .15f), 0);
            Vector3 bottom = new Vector3(at.X, at.Y - (sharkLength * .15f), 0);
            Vector3 tailTop = new Vector3(back.X - (sharkLength * 0.1f), top.Y + (sharkLength * .1f), 0);
            Vector3 tailBottom = new Vector3(tailTop.X, bottom.Y - (sharkLength * .1f), 0);
            Vector3 dorsalTop = new Vector3(top.X, top.Y + (sharkLength * .1f), 0);
            Vector3 dorsalLeftBottom = new Vector3(top.X + (sharkLength * .15f), top.Y - (sharkLength * .1f), 0);
            Vector3 dorsalRightBottom = new Vector3(top.X - (sharkLength * .15f), top.Y - (sharkLength * .1f), 0);
            Vector3 eyeCenter = new Vector3(at.X + (sharkLength * .2f), at.Y + (sharkLength * .05f), 0);
            float eyeRadius = sharkLength * .02f;
            Vector3 mouthTop = new Vector3(at.X + (sharkLength * .3f), eyeCenter.Y - (sharkLength * .1f), 0);
            Vector3 mouthBottom = new Vector3(at.X + (sharkLength * .2f), mouthTop.Y - (sharkLength * .0335f), 0);
            Vector3 mouthBack = new Vector3(at.X + (sharkLength * .13f), at.Y, 0);

            UWB_PrimitiveTriangle sharkFrontHalf = new UWB_PrimitiveTriangle();
            sharkFrontHalf.setBottomLeftPoint(front.X, front.Y, front.Z);
            sharkFrontHalf.setBottomRightPoint(top.X, top.Y, top.Z);
            sharkFrontHalf.setTopPoint(bottom.X, bottom.Y, bottom.Z);
            sharkFrontHalf.setFlatColor(sharkColor);
            sharkFrontHalf.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(sharkFrontHalf);

            UWB_PrimitiveTriangle sharkBackHalf = new UWB_PrimitiveTriangle();
            sharkBackHalf.setBottomLeftPoint(top.X, top.Y, top.Z);
            sharkBackHalf.setBottomRightPoint(back.X, back.Y, back.Z);
            sharkBackHalf.setTopPoint(bottom.X, bottom.Y, bottom.Z);
            sharkBackHalf.setFlatColor(sharkColor);
            sharkBackHalf.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(sharkBackHalf);

            UWB_PrimitiveTriangle tail = new UWB_PrimitiveTriangle();
            tail.setBottomLeftPoint(back.X, back.Y, back.Z);
            tail.setBottomRightPoint(tailTop.X, tailTop.Y, tailTop.Z);
            tail.setTopPoint(tailBottom.X, tailBottom.Y, tailBottom.Z);
            tail.setFlatColor(sharkColor);
            tail.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(tail);

            UWB_PrimitiveTriangle dorsalFin = new UWB_PrimitiveTriangle();
            dorsalFin.setBottomLeftPoint(dorsalLeftBottom.X, dorsalLeftBottom.Y, dorsalLeftBottom.Z);
            dorsalFin.setBottomRightPoint(dorsalRightBottom.X, dorsalRightBottom.Y, dorsalRightBottom.Z);
            dorsalFin.setTopPoint(dorsalTop.X, dorsalTop.Y, dorsalTop.Z);
            dorsalFin.setFlatColor(sharkColor);
            dorsalFin.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(dorsalFin);

            UWB_PrimitiveCircle eye = new UWB_PrimitiveCircle();
            eye.setCenter(eyeCenter);
            eye.setRadius(eyeRadius);
            eye.setFlatColor(new Color(0, 0, 0));
            eye.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(eye);

            UWB_PrimitiveTriangle mouth = new UWB_PrimitiveTriangle();
            mouth.setBottomLeftPoint(mouthTop.X, mouthTop.Y, mouthTop.Z);
            mouth.setBottomRightPoint(mouthBack.X, mouthBack.Y, mouthBack.Z);
            mouth.setTopPoint(mouthBottom.X, mouthBottom.Y, mouthBottom.Z);
            mouth.setFlatColor(sharkMouthColor);
            mouth.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(mouth);

            setPrimitive(mPrimitiveList);
            mXFormInfo.setPivot(at);

            this.pectoralFin = new PectoralFin(at, sharkLength, pectoralFinColor);
            insertChildNode(this.pectoralFin);
        }

        public override void moveNodeByVelocity(float elapsedSecond)
        {
            base.moveNodeByVelocity(elapsedSecond);

            rotateWithVelocity();
        }

        protected override Animation getAnimation(eAnimations animation)
        {
            switch (animation)
            {
                //case eAnimations.Death: return createHitAnimation();
                case eAnimations.Idle: return createSwimAnimation();
            }

            return base.getAnimation(animation);
        }

        /// <summary>
        /// Response to a collision with another SceneObject.
        /// </summary>
        /// <param name="other">The other SceneObject this one collided with.</param>
        /// <param name="collisionPosition">The collision position.</param>
        public override void CollisionResponse(SceneObject other, Vector3 collisionPosition)
        {
            base.CollisionResponse(other, collisionPosition);

            switch (other.getSceneObjectType())
            {
                case eSceneObjectType.Deadly:
                    // reverse other object velocity and X scale
                    Vector3 otherObjectVelocity = other.getVelocity();
                    other.setVelocity(new Vector3(otherObjectVelocity.X * -1, otherObjectVelocity.Y * -1, otherObjectVelocity.Z));

                    UWB_XFormInfo otherXForm = other.getXFormInfo();
                    Vector3 otherObjectScale = otherXForm.getScale();
                    otherXForm.setScale(new Vector3(otherObjectScale.X * -1, otherObjectScale.Y, otherObjectScale.Z));
                    other.setXFormInfo(otherXForm);

                    break;
                case eSceneObjectType.Hero:
                    // TODO: add some creepy half-transparent red blood in the water
                    // TODO: add animation for him chewing (up and down a bit?)

                    break;
                case eSceneObjectType.Safe:
                    // do nothing
                    break;
                default:
                    break;
            }
        }

        private Animation createSwimAnimation()
        {
            //Rotate the fin a bit
            UWB_SceneNode node = findNode("PectoralFin");
            AnimationQue que = new AnimationQue();
            if (node != null)
            {
                //get the current transform of the head
                UWB_XFormInfo targetXform = new UWB_XFormInfo(node.getXFormInfo());

                //Change the rotation
                targetXform.setRotationDegrees(24.0f);

                //Push it onto our stack
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(node.getXFormInfo().getRotationInDegrees());

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));
            }

            return new Animation(que, true);
        }

        #region Public Properties for XML Serialization

        [XmlElement("Pivot")]
        public Vector3 PivotXML
        {
            get
            {
                return this.pivot;
            }
            set
            {
                this.pivot = value;
            }
        }

        [XmlElement("Length")]
        public float LengthXML
        {
            get
            {
                return this.length;
            }
            set
            {
                this.length = value;
            }
        }

        #endregion
    }

    [Serializable]
    public class PectoralFin : UWB_SceneNode
    {
        private Vector3 pivot;
        private float length;
        private Color color;

        public PectoralFin()
            : base()
        {

        }

        public PectoralFin(Vector3 at, float sharkLength, Color color)
        {
            mName = "PectoralFin";

            this.pivot = at;
            this.length = sharkLength;
            this.color = color;

            UWB_PrimitiveList mPrimitiveList = new UWB_PrimitiveList();

            Vector3 upperLeft = new Vector3(at.X - (sharkLength * .16f), at.Y - (sharkLength * .07f), 0);
            Vector3 upperRight = new Vector3(at.X - (sharkLength * .02f), at.Y - (sharkLength * .07f), 0);
            Vector3 bottom = new Vector3(at.X - (sharkLength * .09f), at.Y - (sharkLength * .23f), 0);

            UWB_PrimitiveTriangle pectoralFin = new UWB_PrimitiveTriangle();
            pectoralFin.setBottomLeftPoint(upperLeft.X, upperLeft.Y, upperLeft.Z);
            pectoralFin.setBottomRightPoint(upperRight.X, upperRight.Y, upperRight.Z);
            pectoralFin.setTopPoint(bottom.X, bottom.Y, bottom.Z);
            pectoralFin.setFlatColor(color);
            pectoralFin.setShadeMode(eShadeMode.smFlat);
            mPrimitiveList.append(pectoralFin);

            setPrimitive(mPrimitiveList);
            mXFormInfo.setPivot(new Vector3(((upperLeft.X + upperRight.X) / 2), upperLeft.Y, 0));
        }

        #region Public Properties for XML Serialization

        [XmlElement("Pivot")]
        public Vector3 PivotXML
        {
            get
            {
                return this.pivot;
            }
            set
            {
                this.pivot = value;
            }
        }

        [XmlElement("Length")]
        public float LengthXML
        {
            get
            {
                return this.length;
            }
            set
            {
                this.length = value;
            }
        }

        [XmlElement("Color")]
        public Color ColorXML
        {
            get
            {
                return this.color;
            }
            set
            {
                this.color = value;
            }
        }

        #endregion
    }
}
