using System.Windows.Forms;
using System.Drawing;
using UWBGL_XNA_Lib13;
using System.Xml.Serialization;
using System.IO;
using System;

namespace FinalProject
{
#if WINDOWS || XBOX
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>

        static Model mModel = null;
        //static Model mModel = new Model();
        static private FinalProjectMainForm mMainForm;
        static private ObjectEditorForm mObjectEditor;
        static private bool mPaused = false;

        [STAThread]
        static void Main(string[] args)
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            showNewGameDialog();

            mMainForm = new FinalProjectMainForm();

            //Only create the object editor in debug
            #if DEBUG
                mObjectEditor = new ObjectEditorForm();
            #endif

            Application.Run(mMainForm);
        }

        public static void showWonDialog()
        {
            //Pause the game and show a dialog box with some options
            Program.pause();
            DialogResult result = MessageBox.Show("You have won the game! Do you want to start a new Game?", "Won!", MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                DoNewGame();
            }
        }

        public static void showNewGameDialog()
        {
            //Pause the game and show a dialog box with some options
            Program.pause();
            DialogResult result = MessageBox.Show("Do you want to start a new game?", "Game Start", MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                DoNewGame();
            }
        }

        public static void openObjectEditor(UWB_SceneNode root)
        {
            if (root != null && mObjectEditor != null)
            {
                //pause();
                mObjectEditor.setSceneRoot(root);
                mObjectEditor.Show();
                mObjectEditor.play();
            }
        }

        public static void closeObjectEditor()
        {
            if (mObjectEditor != null)
            {
                //play();
                mObjectEditor.setSceneRoot(null);
                mObjectEditor.Hide();
                mObjectEditor.pause();
            }
        }

        public static void openHelpMenu(string keyword)
        {
            if (mMainForm != null)
            {
                mMainForm.openHelpMenu(keyword);
            }
        }

        /// <summary>
        /// Gets the model.
        /// </summary>
        /// <returns></returns>
        public static Model getModel()
        {
            return mModel;
        }

        public static bool DoNewGame()
        {
            pause();

            //Choose the difficulty
            DialogResult result = MessageBox.Show("Do you want to set the difficulty to Hard?", "Difficulty Level", MessageBoxButtons.YesNo);

            bool loaded = DoLoadGame("scubaDudeOriginalState.xml");
            if (loaded)
            {
                mModel.tempAddToLevel();

                //Set the difficulty based on their choice 
                if (result == DialogResult.Yes)
                    mModel.GameDifficulty = eGameDifficulty.Hard;
                else
                    mModel.GameDifficulty = eGameDifficulty.Easy;
				
                //Pause the game and show a dialog box with starting game info
                MessageBox.Show(mModel.getStory());
            }

            play();

            return loaded;
        }

        public static bool DoLoadGame(string fileName)
        {
            pause();

            //Check if the file exists and if not then return (can't load a file that doesn't exist
            if (!File.Exists(fileName))
                return false;

            //Open the file
            FileStream stream = File.OpenRead(fileName);

            //Read the data from the file
            XmlSerializer serializer = new XmlSerializer(typeof(Model));
            mModel = (Model)serializer.Deserialize(stream);
            mModel.LoadWorld();

            //If we just launched the game this will be null and no need to update the model cause when the form is loaded all that will be done
            //otherwise we need to update the model
            if(mMainForm != null)
                mMainForm.updateModel();

            //Close the file
            stream.Close();

            play();

            return true;
        }

        public static void DoSaveGame(string filePath)
        {
            pause();

            //If it already exists delete it so we can create a new one
            if (File.Exists(filePath))
                File.Delete(filePath);

            //Create a file
            FileStream stream = File.Create(filePath);

            // Convert the object to XML data and put it in the stream.
            XmlSerializer serializer = new XmlSerializer(typeof(Model));
            serializer.Serialize(stream, mModel);

            //Close the file
            stream.Close();

            play();
        }

        public static void onUserPausedGame()
        {
            pause();
            MessageBox.Show("Game Paused");
            play();
        }

        public static void pause()
        {
            if(mMainForm != null)
                mMainForm.pause();

            if(mModel != null)
                mModel.pause();

            mPaused = true;
        }

        public static void play()
        {
            if (mMainForm != null)
                mMainForm.play();

            if (mModel != null)
                mModel.play();

            mPaused = false;
        }

        public static bool isPuased()
        {
            return mPaused;
        }

        public static void updateSceneDropDown()
        {
            if (mMainForm != null)
            {
                mMainForm.updateSceneDropDown();
            }
        }

        public static void updateEquippedItem(Item item)
        {
            if (mMainForm != null)
            {
                mMainForm.updateEquippedItem(item);
            }
        }

        public static void updateGameDifficulty()
        {
            if (mMainForm != null)
            {
                mMainForm.updateGameDifficulty();
            }
        }

        public static void updateUI()
        {
            if (mMainForm != null)
            {
                mMainForm.updateUI();
            }
        }
    }
#endif
}

