﻿using System.Windows.Forms;
using UWBGL_XNA_Lib13;
using Microsoft.Xna.Framework;

namespace FinalProject.UserControls
{
    public partial class XFormInfo : UserControl
    {
        UWB_SceneNode mSceneNode;

        public XFormInfo()
        {
            InitializeComponent();
            mSceneNode = null;

            //Disable z for now
            _sldrZ.Enabled = false;
            _lblZ.Enabled = false;
        }

        public void setSceneNode(UWB_SceneNode sceneNode)
        {
            if (sceneNode == null)
                Visible = false;
            else
                Visible = true;

            mSceneNode = sceneNode;

            updateControls();
        }

        private void updateControls()
        {
            if (mSceneNode != null)
            {
                double min = 0.0;
                double max = 0.0;
                double tickFreq = 0.0;
                double smallChange = 0.0;
                double largeChange = 0.0;
                bool yEnabled = true;
                Vector3 value = Vector3.Zero;

                UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                if (_rbTranslate.Checked)
                {
                    //Update min, max and more for translate (-10 to 10)
                    min = -10.0;
                    max = 10.0;
                    tickFreq = 5.0;
                    smallChange = 1.0;
                    largeChange = 5.0;

                    //Update with current value
                    value.X = xform.getTranslation().X;
                    value.Y = xform.getTranslation().Y;
                    value.Z = xform.getTranslation().Z;
                }
                else if (_rbScale.Checked)
                {
                    //Update min, max and more for scale (-3 to 3)
                    min = -3.0;
                    max = 3.0;
                    tickFreq = 0.5;
                    smallChange = 0.10;
                    largeChange = 0.5;

                    //Update with current value
                    value.X = xform.getScale().X;
                    value.Y = xform.getScale().Y;
                    value.Z = xform.getScale().Z;
                }
                else if (_rbRotate.Checked)
                {
                    //Update min, max and more for rotate (-180 to 180)
                    min = -180.0;
                    max = 180.0;
                    tickFreq = 45.0;
                    smallChange = 10.0;
                    largeChange = 45.0;

                    //Don't support y rotation
                    yEnabled = false;

                    //Update with current value
                    value.X = xform.getRotationInDegrees();
                }
                else if (_rbPivot.Checked)
                {
                    //Update min, max and more translating the pivot (-10 to 10)
                    min = -10.0;
                    max = 10;
                    tickFreq = 5.0;
                    smallChange = 1.0;
                    largeChange = 5.0;

                    value.X = xform.getPivot().X;
                    value.Y = xform.getPivot().Y;
                    value.Z = xform.getPivot().Z;
                }

                //Apply all the values
                _sldrXR.Minimum = min;
                _sldrXR.Maximum = max;
                _sldrXR.TickFrequency = tickFreq;
                _sldrXR.SmallChange = smallChange;
                _sldrXR.LargeChange = largeChange;
                _sldrY.Minimum = min;
                _sldrY.Maximum = max;
                _sldrY.TickFrequency = tickFreq;
                _sldrY.SmallChange = smallChange;
                _sldrY.LargeChange = largeChange;
                _sldrZ.Minimum = min;
                _sldrZ.Maximum = max;
                _sldrZ.TickFrequency = tickFreq;
                _sldrZ.SmallChange = smallChange;
                _sldrZ.LargeChange = largeChange;

                _sldrY.Enabled = yEnabled;

                _sldrXR.Value = value.X;
                _sldrY.Value = value.Y;
                _sldrZ.Value = value.Z;

                _cbShowPivot.Checked = mSceneNode.isPivotVisible();
            }
        }

        private void _cbShowPivot_CheckedChanged(object sender, System.EventArgs e)
        {
            if (mSceneNode != null)
            {
                mSceneNode.setPivotVisible(_cbShowPivot.Checked);
            }
        }

        private void _btnReset_Click(object sender, System.EventArgs e)
        {
            if (mSceneNode != null)
            {
                mSceneNode.resetXFormInfo();
                updateControls();
            }
        }

        private void _rbTranslate_CheckedChanged(object sender, System.EventArgs e)
        {
            updateControls();
        }

        private void _rbScale_CheckedChanged(object sender, System.EventArgs e)
        {
            updateControls();
        }

        private void _rbRotate_CheckedChanged(object sender, System.EventArgs e)
        {
            updateControls();
        }

        private void _rbPivot_CheckedChanged(object sender, System.EventArgs e)
        {
            updateControls();
        }

        private void _sldrXR_TypeValidText(double value)
        {
            if (mSceneNode != null)
            {
                if(_rbTranslate.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 trans = xform.getTranslation();
                    trans.X = (float)value;
                    xform.setTranslation(trans);
                    mSceneNode.setXFormInfo(xform);
                }
                else if (_rbScale.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 scale = xform.getScale();
                    scale.X = (float)value;
                    xform.setScale(scale);
                    mSceneNode.setXFormInfo(xform);
                }
                else if (_rbRotate.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    float rotation = xform.getRotationInDegrees();
                    rotation = (float)value;
                    xform.setRotationDegrees(rotation);
                    mSceneNode.setXFormInfo(xform);
                }
                else if (_rbPivot.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 pivot = xform.getPivot();
                    pivot.X = (float)value;
                    xform.setPivot(pivot);
                    mSceneNode.setXFormInfo(xform);
                }
            }
        }

        private void _sldrY_TypeValidText(double value)
        {
            if (mSceneNode != null)
            {
                if (_rbTranslate.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 trans = xform.getTranslation();
                    trans.Y = (float)value;
                    xform.setTranslation(trans);
                    mSceneNode.setXFormInfo(xform);
                }
                else if (_rbScale.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 scale = xform.getScale();
                    scale.Y = (float)value;
                    xform.setScale(scale);
                    mSceneNode.setXFormInfo(xform);
                }
                else if (_rbPivot.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 pivot = xform.getPivot();
                    pivot.Y = (float)value;
                    xform.setPivot(pivot);
                    mSceneNode.setXFormInfo(xform);
                }
            }
        }

        private void _sldrZ_TypeValidText(double value)
        {
            if (mSceneNode != null)
            {
                if (_rbTranslate.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 trans = xform.getTranslation();
                    trans.Z = (float)value;
                    xform.setTranslation(trans);
                    mSceneNode.setXFormInfo(xform);
                }
                else if (_rbScale.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 scale = xform.getScale();
                    scale.Z = (float)value;
                    xform.setScale(scale);
                    mSceneNode.setXFormInfo(xform);
                }
                else if (_rbPivot.Checked)
                {
                    UWB_XFormInfo xform = mSceneNode.getXFormInfo();
                    Vector3 pivot = xform.getPivot();
                    pivot.Z = (float)value;
                    xform.setPivot(pivot);
                    mSceneNode.setXFormInfo(xform);
                }
            }
        }
    }
}
