﻿using System.Windows.Forms;
using System;
using Microsoft.Xna.Framework;
using NathanScott_Lib;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib13;

namespace FinalProject
{
    public class CameraControlHandler : ModelControlHandler
    {
        #region Private Members

        private MeasurementHandler mMeasurementHandler;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CameraControlHandler"/> class.
        /// </summary>
        public CameraControlHandler(TwoViewModel model):base(model)
        {
            mMeasurementHandler = new MeasurementHandler();
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Draws the graphics.
        /// </summary>
        public override void DrawGraphics()
        {
            if (mModel == null)
                return;

            BeginScene();

            loadW2NDCXform();

            mModel.drawModel();

            mModel.drawZoomedViewBox();

            EndSceneAndShow();
        }

        #endregion

        #region Event Service Methods

        /// <summary>
        /// Called when [mouse button].
        /// </summary>
        /// <param name="down">if set to <c>true</c> [down].</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        public override void OnMouseButton(bool down, MouseEventArgs e)
        {
            Vector3 mousePosition_Device = Vector3.Zero;
            Vector3 mousePosition_World = Vector3.Zero;

            if (mModel != null && GetMousePositions(e, out mousePosition_Device, out mousePosition_World))
            {
                switch (e.Button)
                {
                    case MouseButtons.Left:

                        //change the center of our other view
                        mModel.setZoomedViewCenter(mousePosition_World);
                        break;
                    case MouseButtons.Right:

                        //Put the mouse position into a point for the measurement handler
                        Point2I pos = new Point2I();
                        pos.x = Convert.ToInt32(mousePosition_Device.X);
                        pos.y = Convert.ToInt32(mousePosition_Device.Y);

                        if (down)
                        {
                            //Add the first point to the measurement handler
                            mMeasurementHandler.addPoint(pos);
                        }
                        else //up
                        {
                            //Update the last point to be correct
                            mMeasurementHandler.editLastPoint(pos);

                            //end the measurement
                            mMeasurementHandler.end();

                            //If we have 2 points then calculate the distance apart they are and update our zoom view
                            if (mMeasurementHandler.getCount() > 1)
                            {
                                float x1 = mMeasurementHandler.getPoint(0).x;
                                float x2 = mMeasurementHandler.getPoint(1).x;
                                float distance = x2 - x1;

                                mModel.setZoomedViewZoom(distance);

                                mModel.doneWithZoom();
                            }
                        }

                        break;
                    case MouseButtons.Middle:
                        // reset zoom box
                        if (!down)
                        {
                            mModel.resetZoomedView();
                        }
                        break;
                }

                mModel.setMouseInfo("Large View", new Vector2(e.X, e.Y), mousePosition_Device, mousePosition_World);
            }
        }

        /// <summary>
        /// Raises the <see cref="E:MouseMove"/> event.
        /// </summary>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        public override void OnMouseMove(MouseEventArgs e)
        {
            Vector3 mousePosition_Device = Vector3.Zero;
            Vector3 mousePosition_World = Vector3.Zero;

            if (mModel != null && GetMousePositions(e, out mousePosition_Device, out mousePosition_World))
            {
                switch (e.Button)
                {
                    case MouseButtons.Left:
                        //change the center of our other view
                        mModel.setZoomedViewCenter(mousePosition_World);

                        break;
                    case MouseButtons.Right:

                        //Put the mouse position into a point for the measurement handler
                        Point2I pos = new Point2I();
                        pos.x = Convert.ToInt32(mousePosition_Device.X);
                        pos.y = Convert.ToInt32(mousePosition_Device.Y);

                        //Make sure we only add 1 point and update that point
                        if (mMeasurementHandler.getCount() > 1)
                            mMeasurementHandler.editLastPoint(pos);
                        else
                            mMeasurementHandler.addPoint(pos);

                        //If we have 2 points then calculate the distance apart they are and update our zoom view
                        if (mMeasurementHandler.getCount() > 1)
                        {
                            float x1 = mMeasurementHandler.getPoint(0).x;
                            float x2 = mMeasurementHandler.getPoint(1).x;
                            float distance = x2 - x1;

                            mModel.setZoomedViewZoom(distance);
                        }

                        break;
                    case MouseButtons.Middle:
                        // do nothing
                        break;
                }

                //mModel.setMouseInfo("Large View", new Vector2(e.X, e.Y), mousePosition_Device, mousePosition_World);
                N_Utility.echoToStatusArea(string.Format("moving to: {0}, {1}", mousePosition_World.X, mousePosition_World.Y));
            }
        }

        #endregion

        #region Private Methods

        #endregion
    }
}
