﻿using System.Windows.Forms;
using System;
using Microsoft.Xna.Framework;
using NathanScott_Lib;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib13;

namespace FinalProject
{
    public class ModelControlHandler : UWB_XNAWindowHandler
    {
        #region Private Members

        private UWBGL_UIWindow mWindow;
        protected TwoViewModel mModel;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ModelControlHandler"/> class.
        /// </summary>
        public ModelControlHandler(TwoViewModel model)
        {
            mWindow = new UWBGL_UIWindow(true);
            mModel = model;
        }

        public void setModel(TwoViewModel model)
        {
            mModel = model;
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Draws the graphics.
        /// </summary>
        public override void DrawGraphics()
        {
            if (mModel == null)
                return;

            BeginScene();

            loadW2NDCXform();

            mModel.drawModel();

            EndSceneAndShow();
        }

        /// <summary>Initializes the graphics window.</summary>
        /// <param name="PlaceHolder">The place holder.</param>
        /// <returns>True if operation was successful.</returns>
        public bool Initialize(Control PlaceHolder)
        {
            if (!mWindow.ReplaceFormControl(PlaceHolder.Handle))
                return false;

            if (!mWindow.AttachHandler(this))
                return false;

            return true;
        }

        #endregion

        #region Event Service Methods

        #endregion

        #region Private Methods

        /// <summary>
        /// Gets the mouse positions in device and world coordinates.
        /// </summary>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        /// <param name="mousePosition_Device">The mouse click position_ device.</param>
        /// <param name="mousePosition_World">The mouse click position_ world.</param>
        /// <returns>true if successful, otherwise false.</returns>
        protected bool GetMousePositions(MouseEventArgs e, out Vector3 mousePosition_Device, out Vector3 mousePosition_World)
        {
            int deviceX = 0, deviceY = 0;
            hardwareToDevice(e.X, e.Y, ref deviceX, ref deviceY);

            float worldX = 0, worldY = 0;
            DeviceToWorld(deviceX, deviceY, ref worldX, ref worldY);

            mousePosition_Device = new Vector3((float)deviceX, (float)deviceY, 0);
            mousePosition_World = new Vector3(worldX, worldY, 0);

            return true;
        }

        #endregion
    }
}
