﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace FinalProject.Windows
{
    /// <summary>
    /// Requires at least one table layout panel
    /// </summary>
    public partial class InventoryForm : Form
    {
        /// <summary>
        /// This is called each loop in the populate control so you can
        /// customize your button or whatever else you want to do each loop
        /// Note: Make sure and set click callback
        /// </summary>
        /// <param name="button">button to be customized</param>
        /// <param name="item">item to use to customize it with/it corresponds to</param>
        protected delegate void ItemButtonCreatedCallback(Button button, Item item);

        public InventoryForm()
        {
            InitializeComponent();
        }

        protected override void OnPaint(PaintEventArgs pe)
        {
            base.OnPaint(pe);
        }

        protected void updateTable(TableLayoutPanel table, Inventory list, ItemButtonCreatedCallback createdCallback, bool checkItems)
        {
            //Clear out the table and stuff that was there before
            table.Controls.Clear();
            table.ColumnStyles.Clear();
            table.RowStyles.Clear();
            table.RowCount = 0;
            table.ColumnCount = 0;

            for (int i = 0; i < list.getItemCount(); i++)
            {
                //Create a button and associate with our item
                Item item = list.getItem(i);

                if (!checkItems || (checkItems && itemCheck(item)))
                {
                    Button button = new Button();
                    button.Tag = item;
                    button.Width = 100;
                    button.Height = 32;

                    //Do anything custom
                    if (createdCallback != null)
                        createdCallback(button, item);

                    //Add it to our table and handle the table
                    table.Controls.Add(button);
                    table.ColumnCount++;
                    table.ColumnStyles.Add(new ColumnStyle());
                }
            }
        }

        protected virtual bool itemCheck(Item item)
        {
            return true;
        }
    }
}
