﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using FinalProject.Windows;

namespace FinalProject
{
    public partial class InventoryWindow : InventoryForm
    {
        //Callbacks
        public delegate void InventoryCloseEventHanlder();

        public event InventoryCloseEventHanlder InventoryCloseCallback;

        private Hero mPerson = null;
        private Item mEquiptedItem = null;
        private Button mEquipted = null;

        public InventoryWindow()
        {
            InitializeComponent();
        }

        public void ini(Hero person)
        {
            mPerson = person;

            mEquiptedItem = mPerson.getEquippedItem();

            updateTable(_tblInventory, person.Inventory, onButtonCreated, true);
        }

        protected override bool itemCheck(Item item)
        {
            return item.isEquippable();
        }

        /// <summary>
        /// This is called each loop in the populate control so you can
        /// customize your button or whatever else you want to do each loop
        /// </summary>
        /// <param name="button">button to be customized</param>
        /// <param name="item">item to use to customize it with/it corresponds to</param>
        private void onButtonCreated(Button button, Item item)
        {
            //set the button callback
            button.Click += new EventHandler(this.onButtonClicked);

            if (ReferenceEquals(mEquiptedItem, button.Tag))
            {
                button.Enabled = false;
                mEquipted = button;
            }

            //Customize our button
            button.BackgroundImage = item.PreviewImage;
            button.BackgroundImageLayout = ImageLayout.Zoom;
            button.Text = item.getInventoryText();
            button.TextAlign = ContentAlignment.BottomRight;
        }

        /// <summary>
        /// Called when a button is clicked
        /// </summary>
        /// <param name="sender">button instance that was clicked</param>
        /// <param name="e">other args</param>
        private void onButtonClicked(object sender, EventArgs e)
        {
            if (sender is Button)
            {
                Button button = sender as Button;

                if (button.Tag is Item)
                {
                    Item item = button.Tag as Item;

                    Item lastEquipped = null;
                    //This handles checking the money and subtracting it
                    if (mPerson.setEquippedItem(item, ref lastEquipped))
                    {
                        //Set the old item as un-equipped
                        if(mEquipted != null)
                            mEquipted.Enabled = true;

                        //Equip the new item
                        mEquiptedItem = item;
                        mEquipted = button;
                        button.Enabled = false;
                    }
                    else
                    {
                        string message = "This item can't be equipped.";
                        MessageBox.Show(message);
                    }
                }
            }
        }

        private void _btDone_Click(object sender, EventArgs e)
        {
            if (this.InventoryCloseCallback != null)
                this.InventoryCloseCallback();

            Close();
        }
    }
}
