﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace NathanScott_Lib
{
    public static class Nate_and_Scott_Collision
    {
        public static bool intersectBoxBox(Vector3 box1Min, Vector3 box1Max, Vector3 box2Min, Vector3 box2Max, ref Vector3 pos)
        {
            bool xOutside = box1Min.X > box2Max.X || box1Max.X < box2Min.X;
            if (xOutside)
                return false;

            bool yOutside = box1Min.Y > box2Max.Y || box1Max.Y < box2Min.Y;
            if (yOutside)
                return false;

            bool zOutside = box1Min.Z > box2Max.Z || box1Max.Z < box2Min.Z;
            if (zOutside)
                return false;

            // There is an intersection, now we must approximate a collision point
            // find the bigger box, then use the smaller one and take the mid-point of that one, but set to the other's edge!

            // if box 1 is smaller, use it, else use box 2.  check both boxes for both x and y
            if ((box1Min.X > box2Min.X) && (box1Max.X < box2Max.X))
            {
                // Box 1 is narrower than box 2 - AND, we are colliding the top or bottom of box 1, so set
                // the X to be in the middle of box 1, then set the appropriate Y.

                pos = Vector3.Zero;

                pos.X = (box1Min.X + box1Max.X) / 2;

                if (box1Max.Y > box2Min.Y)
                {
                    // colliding with top edge of box 1, so set to the bottom edge of box 2
                    //pos.Y = box1Max.Y;
                    pos.Y = box2Min.Y;
                }
                else
                {
                    // colliding with bottom edge of box 1, so set to the top edge of box 2
                    //pos.Y = box1Min.Y;
                    pos.Y = box2Max.Y;
                }
            }
            else if ((box2Min.X > box1Min.X) && (box2Max.X < box1Max.X))
            {
                // Box 2 is narrower than box 1 - AND, we are colliding the top or bottom of box 2, so set
                // the X to be in the middle of box 2, then set the appropriate Y.

                pos = Vector3.Zero;

                pos.X = (box2Min.X + box2Max.X) / 2;

                if (box2Max.Y > box1Min.Y)
                {
                    // colliding with top edge of box 2, so set to the bottom edge of box 1
                    //pos.Y = box2Max.Y;
                    pos.Y = box1Min.Y;
                }
                else
                {
                    // colliding with bottom edge of box 2, so set to the top edge of box 1
                    //pos.Y = box2Min.Y;
                    pos.Y = box1Max.Y;
                }
            }
            else if ((box1Min.Y > box2Min.Y) && (box1Max.Y < box2Max.Y))
            {
                // Box 1 is shorter than box 2, and we are colliding the left or right of box 1, so set 
                // the Y to be in the middle of box 1, then set the appropriate X.

                pos = Vector3.Zero;

                pos.Y = (box1Min.Y + box1Max.Y) / 2;

                if (box1Max.X > box2Min.X)
                {
                    // colliding with right edge of box 1, so set to the left edge of box 2
                    //pos.X = box1Max.X;
                    pos.X = box2Min.X;
                }
                else
                {
                    // colliding with left edge of box 1, so set to the right edge of box 2
                    //pos.X = box1Min.X;
                    pos.X = box2Max.X;
                }
            }
            else if ((box2Min.Y > box1Min.Y) && (box2Max.Y < box1Max.Y))
            {
                // Box 2 is shorter than box 1, and we are colliding the left or right of box 2, so set 
                // the Y to be in the middle of box 2, then set the appropriate X.

                pos = Vector3.Zero;

                pos.Y = (box2Min.Y + box2Max.Y) / 2;

                if (box2Max.X > box1Min.X)
                {
                    // colliding with right edge of box 2, so set to the left edge of box 1
                    //pos.X = box2Max.X;
                    pos.X = box1Min.X;
                }
                else
                {
                    // colliding with left edge of box 2, so set to the right edge of box 1
                    //pos.X = box2Min.X;
                    pos.X = box1Max.X;
                }
            }

            return true;
        }

        public static bool intersectSphereSphere(Vector3 center1, float radius1, Vector3 center2, float radius2, ref Vector3 pos)
        {
            Vector3 v = center2 - center1;
            float vLength = v.Length();

            if (vLength < (radius1 + radius2))
            {
                Vector3 middle = (center1 + center2) / 2f;
                float diff = radius2 - radius1;
                float distance = Vector3.Distance(center1, center2);
                float k = diff/distance;
                pos = N_Math.getPointOnLine(center1, center2, middle, diff);
                return true;
            }
            else
                return false;
        }

        public static bool intersectBoxSphere(Vector3 boxMin, Vector3 boxMax, Vector3 center, float radius, ref Vector3 pos)
        {
            //Find the point from the box that is closest to the sphere's center
            Vector3 closest = new Vector3(0, 0, 0);

            if (center.X < boxMin.X)
                closest.X = boxMin.X;
            else if (center.X > boxMax.X)
                closest.X = boxMax.X;
            else
                closest.X = center.X;

            if (center.Y < boxMin.Y)
                closest.Y = boxMin.Y;
            else if (center.Y > boxMax.Y)
                closest.Y = boxMax.Y;
            else
                closest.Y = center.Y;

            if (center.Z < boxMin.Z)
                closest.Z = boxMin.Z;
            else if (center.Z > boxMax.Z)
                closest.Z = boxMax.Z;
            else
                closest.Z = center.Z;

            //check if the distance from the closest point to the sphere's center is less then the sphere's radius
            if (Vector3.Distance(closest, center) < radius)
            {
                pos = closest;
                return true;
            }
            else
                return false;
        }

        public static bool isOutsideBox(Vector3 inBoxMin, Vector3 inBoxMax, Vector3 outBoxMin, Vector3 outBoxMax)
        {
            bool xOutside = inBoxMin.X > outBoxMax.X || inBoxMax.X < outBoxMin.X;
            bool yOutside = inBoxMin.Y > outBoxMax.Y || inBoxMax.Y < outBoxMin.Y;

            return (xOutside || yOutside);
        }
    }
}
