﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Linq;
using System.Text;

namespace NathanScott_Lib
{
    public class N_Utility
    {
        private static List<ToolStripLabel> mEchoTextBoxs = new List<ToolStripLabel>();
        private static readonly Random random = new Random();
        private static readonly object syncLock = new object();

        public static void addEchoTextBox(ToolStripLabel echoBox)
        {
            mEchoTextBoxs.Add(echoBox);
        }

        public static void echoToStatusArea(string echoString)
        {
            echoToStatusArea(echoString, 0);
        }

        public static void echoToStatusArea(string echoString, int index)
        {
            if (index >= 0 && index < mEchoTextBoxs.Count && mEchoTextBoxs[index] != null)
                mEchoTextBoxs[index].Text = echoString;
        }

        public static void setToStatusAreaColor(System.Drawing.Color color)
        {
            setToStatusAreaColor(color, 0);
        }

        public static void setToStatusAreaColor(System.Drawing.Color color, int index)
        {
            if (index >= 0 && index < mEchoTextBoxs.Count && mEchoTextBoxs[index] != null)
                mEchoTextBoxs[index].ForeColor = color;
        }

        /// <summary>
        /// Generates a random int.
        /// </summary>
        /// <param name="min">The min.</param>
        /// <param name="max">The max.</param>
        /// <returns></returns>
        public static int GenerateRandomInt(int min, int max)
        {
            lock (syncLock)
            { // synchronize
                return random.Next(min, max);
            }
        }

        /// <summary>
        /// Generates a random float between min and max.
        /// </summary>
        /// <param name="min">The min.</param>
        /// <param name="max">The max.</param>
        /// <exception cref="InvalidOperationException">Min must be less than or equal to Max.</exception>
        /// <returns>a random float between min and max.</returns>
        public static float GenerateRandomFloat(float min, float max)
        {
            // sanity check
            if (min > max)
            {
                throw new InvalidOperationException("Max must be greater than or equal to Min");
            }

            // This next little bit of code works because of the following:
            // The range is the difference of the max and the min.
            // Random.NextDouble() * range -> this results in a number between zero and range.
            // Then, if you add the min, you should always have a number between min and max,
            // including positive and negative.
            float range = max - min;
            lock (syncLock)
            { // synchronize
                return ((float)random.NextDouble() * range) + min;
            }
        }

        public static double randomNumber()
        {
            Random rand = new Random();
            return rand.NextDouble();
        }

        public static double randomNumber(double range)
        {
            Random rand = new Random();
            return range * rand.NextDouble();
        }

        public static float randomNumber(float range)
        {
            Random rand = new Random();
            return range * (float)rand.NextDouble();
        }

        public static double randomNumber(double minRange, double maxRange)
        {
            return N_Math.fitToRange(randomNumber(), minRange, maxRange);
        }

        public static float randomNumber(float minRange, float maxRange)
        {
            return N_Math.fitToRange((float)randomNumber(), minRange, maxRange);
        }
    }
}
