﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;


namespace NathanScott_Lib.CustomControls
{
    public partial class ColorSliderControl : UserControl
    {
        public ColorSliderControl()
        {
            InitializeComponent();

            updateDimentions();
        }

        private Color mColor;

        /// <summary>
        /// Event handler argument type for valid color value callback
        /// </summary>
        /// <param name="color">new color</param>
        public delegate void TypeValidColorEventHandler(Color color);

        //[SRDescription("ValidColor")]
        public event TypeValidColorEventHandler ValidColor;

        public Color Value
        {
            get
            {
                return mColor;
            }

            set
            {
                mColor = value;

                _sldrR.Value = N_Math.colorByteToDouble(mColor.R);
                _sldrG.Value = N_Math.colorByteToDouble(mColor.G);
                _sldrB.Value = N_Math.colorByteToDouble(mColor.B);
                _sldrA.Value = N_Math.colorByteToDouble(mColor.A);
            }
        }

        /// <summary>
        /// Get/set red value
        /// </summary>
        public byte R
        {
            get
            {
                return mColor.R;
            }

            set
            {
                mColor.R = value;
                _sldrR.Value = N_Math.colorByteToDouble(mColor.R);
            }
        }

        /// <summary>
        /// Get/set green value
        /// </summary>
        public byte G
        {
            get
            {
                return mColor.G;
            }

            set
            {
                mColor.G = value;
                _sldrG.Value = N_Math.colorByteToDouble(mColor.G);
            }
        }

        /// <summary>
        /// Get/set blue value
        /// </summary>
        public byte B
        {
            get
            {
                return mColor.B;
            }

            set
            {
                mColor.B = value;
                _sldrB.Value = N_Math.colorByteToDouble(mColor.B);
            }
        }

        /// <summary>
        /// Get/set alpha value
        /// </summary>
        public byte A
        {
            get
            {
                return mColor.A;
            }

            set
            {
                mColor.A = value;
                _sldrA.Value = N_Math.colorByteToDouble(mColor.A);
            }
        }

        /// <summary>
        /// Get/Set small change amount for slider
        /// </summary>
        public double SmallChange
        {
            get
            {
                //only need one since they will all be the same
                return _sldrR.SmallChange;
            }

            set
            {
                _sldrR.SmallChange = value;
                _sldrG.SmallChange = value;
                _sldrB.SmallChange = value;
                _sldrA.SmallChange = value;
            }
        }

        /// <summary>
        /// Get/Set Large change amount for slider
        /// </summary>
        public double LargeChange
        {
            get
            {
                //only need one since they will all be the same
                return _sldrR.LargeChange;
            }

            set
            {
                _sldrR.LargeChange = value;
                _sldrG.LargeChange = value;
                _sldrB.LargeChange = value;
                _sldrA.LargeChange = value;
            }
        }

        /// <summary>
        /// Get/Set Tick Style for the slider
        /// </summary>
        public TickStyle TickStyle
        {
            get
            {
                //only need one since they will all be the same
                return _sldrR.TickStyle;
            }

            set
            {
                _sldrR.TickStyle = value;
                _sldrG.TickStyle = value;
                _sldrB.TickStyle = value;
                _sldrA.TickStyle = value;
            }
        }

        /// <summary>
        /// Get/Set Tick Frequency for the slider
        /// </summary>
        public double TickFrequency
        {
            get
            {
                //only need one since they will all be the same
                return _sldrR.TickFrequency;
            }

            set
            {
                _sldrR.TickFrequency = value;
                _sldrG.TickFrequency = value;
                _sldrB.TickFrequency = value;
                _sldrA.TickFrequency = value;
            }
        }

        /// <summary>
        /// Called when red slider is dragged or new valid number is entered into the textbox.
        /// </summary>
        /// <param name="value">The value.</param>
        private void _sldrR_TypeValidText(double value)
        {
            mColor.R = N_Math.colorDoubleToByte(value);
            if (this.ValidColor != null)
                this.ValidColor(mColor);
        }

        /// <summary>
        /// Called when green slider is dragged or new valid number is entered into the textbox.
        /// </summary>
        /// <param name="value">The value.</param>
        private void _sldrG_TypeValidText(double value)
        {
            mColor.G = N_Math.colorDoubleToByte(value);
            if (this.ValidColor != null)
                this.ValidColor(mColor);
        }

        /// <summary>
        /// Called when blue slider is dragged or new valid number is entered into the textbox.
        /// </summary>
        /// <param name="value">The value.</param>
        private void _sldrB_TypeValidText(double value)
        {
            mColor.B = N_Math.colorDoubleToByte(value);
            if (this.ValidColor != null)
                this.ValidColor(mColor);
        }

        /// <summary>
        /// Called when alpha slider is dragged or new valid number is entered into the textbox.
        /// </summary>
        /// <param name="value">The value.</param>
        private void _sldrA_TypeValidText(double value)
        {
            mColor.A = N_Math.colorDoubleToByte(value);
            if (this.ValidColor != null)
                this.ValidColor(mColor);
        }

        private void ColorSliderControl_Resize(object sender, EventArgs e)
        {
            updateDimentions();
        }

        private void updateDimentions()
        {
            int count = Controls.Count;
            int spacing = (Height / (count/2));
            int labelY = 0;
            int sliderY = 0;
            foreach (Control ctrl in Controls)
            {
                int width = ctrl.Width;
                int height = ctrl.Height;
                int y = ctrl.Location.Y;
                if (ctrl is Label)
                {
                    y = labelY;
                    labelY += spacing;
                }
                else if (ctrl is SliderWithTextBox.SliderWithTextBoxControl)
                {
                    y = sliderY;
                    sliderY += spacing;
                    height = spacing;
                }
                ctrl.SetBounds(ctrl.Location.X, y, width, height);
            }
        }
    }
}
