﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace NathanScott_Lib.CustomControls
{
    public partial class Point2TextBox : UserControl
    {
        private bool mReadOnly;
        private uint mDecimalPlaces;
        private Point2D mValue;
        private bool mInValid;

        public Point2TextBox()
        {
            InitializeComponent();

            mDecimalPlaces = 0;
            mValue = new Point2D(0, 0);
            mReadOnly = true;
            TextBox1.ReadOnly = mReadOnly;
            TextBox2.ReadOnly = mReadOnly;
            mInValid = false;
        }

        /// <summary>
        /// Gets/Sets number of decimal places to display
        /// </summary>
        public uint NumberOfDecimalPlaces
        {
            get
            {
                return mDecimalPlaces;
            }

            set
            {
                mDecimalPlaces = value;
                updateTextBoxes();
            }
        }

        /// <summary>
        /// Gets/Sets if the user can enter text into the textboxes
        /// </summary>
        public bool ReadOnly
        {
            get
            {
                return mReadOnly;
            }

            set
            {
                mReadOnly = value;
                TextBox1.ReadOnly = mReadOnly;
                TextBox2.ReadOnly = mReadOnly;
            }
        }

        /// <summary>
        /// Gets/Sets if the user can enter text into the textboxes
        /// </summary>
        public bool Invalid
        {
            get
            {
                return mInValid;
            }

            set
            {
                mInValid = value;
                updateTextBoxes();
            }
        }

        /// <summary>
        /// Gets/Sets the first value of to be displayed
        /// </summary>
        public double Value1
        {
            get
            {
                return mValue.x;
            }

            set
            {
                mValue.x = value;
                updateTextBoxes();
            }
        }

        /// <summary>
        /// Gets/Sets the first value of to be displayed
        /// </summary>
        public double Value2
        {
            get
            {
                return mValue.y;
            }

            set
            {
                mValue.y = value;
                updateTextBoxes();
            }
        }

        /// <summary>
        /// Gets/Sets the label for the first element
        /// </summary>
        public String Lable1
        {
            get
            {
                return Label1.Text;
            }

            set
            {
                Label1.Text = value;
            }
        }

        /// <summary>
        /// Gets/Sets the label for the second element
        /// </summary>
        public String Lable2
        {
            get
            {
                return Label2.Text;
            }

            set
            {
                Label2.Text = value;
            }
        }

        private void updateTextBoxes()
        {
            if (mInValid)
            {
                TextBox1.Text = "---";
                TextBox2.Text = "---";
            }
            else
            {
                string format = "N" + mDecimalPlaces;
                TextBox1.Text = mValue.x.ToString(format);
                TextBox2.Text = mValue.y.ToString(format);
            }
        }
    }
}
