﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace UWBGL_XNA_Lib13
{
    public class UWB_ParticleCircle : UWB_PrimitiveCircle
    {
        /// <summary>
        /// Current life in seconds
        /// </summary>
        private float mLife;

        /// <summary>
        /// Total life of this particle
        /// </summary>
        private float mLifeTime;

        /// <summary>
        /// Color when particle was first created
        /// </summary>
        private Color mStartColor;

        /// <summary>
        /// Create a particle circle with a life
        /// </summary>
        /// <param name="life">life of the particle in seconds</param>
        public UWB_ParticleCircle(float life, Color color)
		{
			mCenter = Vector3.Zero;
			mRadius = 0;
            mBounds = new UWB_BoundingBox();
            mLifeTime = mLife = life;
            mStartColor = color;
            setFlatColor(color);
            setShadeMode(eShadeMode.smFlat);
		}

        /// <summary>
        /// Check if particle life is over (0)
        /// </summary>
        /// <returns>true if particle is dead, false if not</returns>
        public bool isDead()
        {
            return (mLife <= 0f);
        }

        /// <summary>
        /// Move the particle based on it's velocity and seconds passed,
        /// change the color of it based on it's life (half way in life fade out).
        /// </summary>
        /// <param name="elapsedSeconds">seconds passed since last update</param>
        public override void update(float elapsedSeconds)
		{
            base.update(elapsedSeconds);

            //Update the life of the particle
            mLife = Math.Max(mLife - elapsedSeconds, 0f);

            float ratio = mLife / mLifeTime;

            Color color = new Color(192, 192, 192, 0);

            //If less than half way in the lifetime interpolate to background color
            //else pick random color in the between red and yellow
            if (ratio <= 0.5f)
            {
                color = Color.Lerp(color, mStartColor, ratio / 0.5f);
            }
            else
            {
                color = Color.Lerp(Color.Red, Color.Yellow, UWB_Utility.UWB_RandomNumber(0.0f, 1.0f));
            }

            setFlatColor(color);
		}
    }
}
