﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using NathanScott_Lib;

namespace UWBGL_XNA_Lib13
{
    public class UWB_PrimitivePolygon : UWB_Primitive
    {
        protected Vector3 mCenter;
        protected float mRadius;
        protected UWB_BoundingSphere mBounds;
        protected bool mUseBestBounds;

        public UWB_PrimitivePolygon()
        {
            mCenter = Vector3.Zero;
            mRadius = 0;
            mBounds = new UWB_BoundingSphere();
            useBestBounds(false);
        }

        public UWB_PrimitivePolygon(bool createBestBounds)
        {
            mCenter = Vector3.Zero;
            mRadius = 0;
            mBounds = new UWB_BoundingSphere();
            useBestBounds(createBestBounds);
        }

        public bool hasBestBounds()
        {
            return mUseBestBounds;
        }

        public void useBestBounds(bool val)
        {
            if (mUseBestBounds != val)
            {
                mUseBestBounds = val;

                if (mUseBestBounds)
                {
                    mBounds.setCenterAndRadius(mCenter, mRadius);
                }
                else
                {
                    mBounds.setCenterAndRadius(mCenter, mRadius * 0.75f);
                }
            }
        }

        protected override void setupDrawAttributes(UWB_DrawHelper drawHelper)
        {
            base.setupDrawAttributes(drawHelper);
            drawHelper.setColor1(mFlatColor);
            drawHelper.setColor2(mShadingColor);
            drawHelper.setShadeMode(mShadeMode);
            drawHelper.setFillMode(mFillMode);
        }

        protected override void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
        {
            eLevelofDetail oldLod = drawHelper.getLod();
            drawHelper.setLod(lod);
            drawHelper.drawPolygon(mCenter, mRadius);
            drawHelper.setLod(oldLod);
        }

        public override void mouseDownVertex(int vertexID, float x, float y)
        {
            if (vertexID == 0)
            {
                setCenter(x, y, 0f);
                setRadius(0);
            }
            else
            {
                float dx = x - mCenter.X;
                float dy = y - mCenter.Y;
                float radius = (float)Math.Sqrt(dx * dx + dy * dy);
                setRadius(radius);
            }
        }

        public override void moveTo(float x, float y)
        {
            setCenter(new Vector3(x, y, 0.0f));
        }

        public void setCenter(Vector3 center)
        {
            mCenter = center;
            mBounds.setCenter(mCenter);
        }

        public void setCenter(float x, float y, float z)
        {
            mCenter.X = x;
            mCenter.Y = y;
            mCenter.Z = z;
            mBounds.setCenter(mCenter);
        }

        public override Vector3 getLocation()
        {
            return mCenter;
        }

        public void setRadius(float radius)
        {
            mRadius = radius;

            if (mUseBestBounds)
            {
                mBounds.setRadius(mRadius);
            }
            else
            {
                mBounds.setRadius(mRadius * 0.75f);
            }
        }

        public float getRadius()
        {
            return mRadius;
        }

        public override float getVolume()
        {
            return ((4f * N_Math.PI * (float)Math.Pow(mRadius, 3f)) / 3f);
        }

        public override void update(float elapsedSeconds)
        {
            Vector3 adjVelocity = mVelocity * elapsedSeconds;
            mCenter += adjVelocity;
            mBounds.setCenter(mCenter);
        }

        public override UWB_BoundingVolume getBoundingVolume(eLevelofDetail lod)
        {
            return mBounds;
        }
    }
}
