using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace UWBGL_XNA_Lib13
{
	public class UWB_PrimitiveRectangle : UWB_Primitive
	{
        protected UWB_BoundingVolume mBounds;
        protected Vector3 mMin;
        protected Vector3 mMax;

        public UWB_PrimitiveRectangle()
        {
            mMin = Vector3.Zero;
            mMax = Vector3.Zero;
			mBounds = new UWB_BoundingBox(); // default, but can be changed.
		}

        public void useBoundingSphereCollectionForVolume(bool useSphereList)
        {
            if (useSphereList)
            {
                mBounds = new UWB_BoundingBox_SphereList(mMin, mMax);
            }
            else
            {
                mBounds = new UWB_BoundingBox();
                ((UWB_BoundingBox)mBounds).setCorners(mMin, mMax);
            }
        }

        public void setCorners(Vector3 corner1, Vector3 corner2)
        {
            mMin.X = corner1.X < corner2.X ? corner1.X : corner2.X;
            mMin.Y = corner1.Y < corner2.Y ? corner1.Y : corner2.Y;
            mMin.Z = corner1.Z < corner2.Z ? corner1.Z : corner2.Z;

            mMax.X = corner1.X > corner2.X ? corner1.X : corner2.X;
            mMax.Y = corner1.Y > corner2.Y ? corner1.Y : corner2.Y;
            mMax.Z = corner1.Z > corner2.Z ? corner1.Z : corner2.Z;

            updateBounds();
        }

        public void define(Vector3 center, float width, float height)
        {
            setCorners(new Vector3(center.X - (width / 2), center.Y - (height / 2), 0),
                       new Vector3(center.X + (width / 2), center.Y + (height / 2), 0));

            updateBounds();
        }

        protected float getLargestEdge()
        {
            Vector3 edgeSizes = mMax - mMin;
            edgeSizes.X = Math.Abs(edgeSizes.X);
            edgeSizes.Y = Math.Abs(edgeSizes.Y);
            edgeSizes.Z = Math.Abs(edgeSizes.Z);
            return Math.Max(Math.Max(edgeSizes.X, edgeSizes.Y), edgeSizes.Z);
        }

        public override float getVolume()
        {
            Vector3 edgeSizes = mMax - mMin;
            edgeSizes.X = Math.Abs(edgeSizes.X);
            edgeSizes.Y = Math.Abs(edgeSizes.Y);
            edgeSizes.Z = Math.Max(Math.Abs(edgeSizes.Z), 1);
            return edgeSizes.X * edgeSizes.Y * edgeSizes.Z;
        }

        protected override void setupDrawAttributes(UWB_DrawHelper drawHelper)
        {
            base.setupDrawAttributes(drawHelper);
            drawHelper.setColor1(mFlatColor);
            drawHelper.setColor2(mShadingColor);
            drawHelper.setShadeMode(mShadeMode);
            drawHelper.setFillMode(mFillMode);
        }

        protected override void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
        {
            if (drawHelper != null)
            {
                drawHelper.drawRectangle(mMin, mMax, mTextureScale, mUseTextureAspectRatio);
            }
        }

        public override void mouseDownVertex(int vertexID, float x, float y)
        {
            if (vertexID == 0)
                setMin(x, y, 0f);

            setMax(x, y, 0f);
        }

        public override void moveTo(float x, float y)
        {
            Vector3 center = getMidPoint();
            float xDist = x - center.X;
            float yDist = y - center.Y;

            setMin(mMin.X + xDist, mMin.Y + yDist, 0.0f);
            setMax(mMax.X + xDist, mMax.Y + yDist, 0.0f);

            updateBounds();
        }

        private Vector3 getMidPoint()
        {
            Vector3 m = (mMin + mMax) / 2f;
            return m;
        }

        public override Vector3 getLocation()
        {
            return getMidPoint();
        }

        public void setMin(float x, float y, float z)
        {
            mMin.X = x;
            mMin.Y = y;
            mMin.Z = z;
        }

        public Vector3 getMin()
        {
            return mMin;
        }

        public void setMax(float x, float y, float z)
        {
            mMax.X = x;
            mMax.Y = y;
            mMax.Z = z;
        }

        public Vector3 getMax()
        {
            return mMax;
        }
		
		public float getWidth()
        {
            return mMax.X - mMin.X;
        }
		
		public float getHeight()
        {
            return mMax.Y - mMin.Y;
        }

        public void setWidth(float width)
        {
            float oldWidth = getWidth();
            float diff = width - oldWidth;

            mMax.X += diff / 2;
            mMin.X -= diff / 2;
        }

        public void setHeight(float height)
        {
            float oldHeight = getHeight();
            float diff = height - oldHeight;

            mMax.Y += diff / 2;
            mMin.Y -= diff / 2;
        }

        public override void update(float elapsedSeconds)
        {
            Vector3 adjVelocity = mVelocity * elapsedSeconds;
            mMin += adjVelocity;
            mMax += adjVelocity;
        }

		public override UWB_BoundingVolume getBoundingVolume(eLevelofDetail lod)
		{
            updateBounds();

			return mBounds;
		}

        /// <summary>
        /// Updates the bounds.
        /// </summary>
        private void updateBounds()
        {
            if (mBounds != null)
            {
                if (mBounds is UWB_BoundingBox)
                {
                    UWB_BoundingBox temp = mBounds as UWB_BoundingBox;
                    temp.setCorners(mMin, mMax);
                }
                else if (mBounds is UWB_BoundingBox_SphereList)
                {
                    UWB_BoundingBox_SphereList temp = mBounds as UWB_BoundingBox_SphereList;
                    temp = new UWB_BoundingBox_SphereList(mMin, mMax);
                }
            }
        }
	}
}
