﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace UWBGL_XNA_Lib13
{
    public class UWB_PrimitiveTriangle : UWB_Primitive
    {
        protected Vector3 mTop;
        protected Vector3 mBottomLeft;
        protected Vector3 mBottomRight;
        protected UWB_BoundingSphere mBounds;

        public UWB_PrimitiveTriangle()
        {
            mTop = Vector3.Zero;
            mBottomLeft = Vector3.Zero;
            mBottomRight = Vector3.Zero;
            mBounds = new UWB_BoundingSphere();
        }

        protected override void setupDrawAttributes(UWB_DrawHelper drawHelper)
        {
            base.setupDrawAttributes(drawHelper);
            drawHelper.setColor1(mFlatColor);
            drawHelper.setColor2(mShadingColor);
            drawHelper.setShadeMode(mShadeMode);
            drawHelper.setFillMode(mFillMode);
        }

        protected override void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
        {
            drawHelper.drawTriangle(mTop, mBottomRight, mBottomLeft);
        }

        public override void mouseDownVertex(int vertexID, float x, float y)
        {
            if (vertexID == 0)
                setTopPoint(x, y, 0f);

            if (vertexID == 1)
                setBottomRightPoint(x, y, 0f);

            setBottomLeftPoint(x, y, 0f);
        }

        public override void moveTo(float x, float y)
        {
            Vector3 center = getCenter();
            float xDist = x - center.X;
            float yDist = y - center.Y;

            setTopPoint(mTop.X + xDist, mTop.Y + yDist, 0.0f);
            setBottomLeftPoint(mBottomLeft.X + xDist, mBottomLeft.Y + yDist, 0.0f);
            setBottomRightPoint(mBottomRight.X + xDist, mBottomRight.Y + yDist, 0.0f);
        }

        private Vector3 getCenter()
        {
            Vector3 m = (mTop + mBottomRight + mBottomLeft) / 3f;
            return m;
        }

        public override Vector3 getLocation()
        {
            return getCenter();
        }

        public void setTopPoint(float x, float y, float z)
        {
            mTop.X = x;
            mTop.Y = y;
            mTop.Z = z;
        }

        public Vector3 getTopPoint()
        {
            return mTop;
        }

        public void setBottomLeftPoint(float x, float y, float z)
        {
            mBottomLeft.X = x;
            mBottomLeft.Y = y;
            mBottomLeft.Z = z;
        }

        public Vector3 getBottomLeftPoint()
        {
            return mBottomLeft;
        }

        public void setBottomRightPoint(float x, float y, float z)
        {
            mBottomRight.X = x;
            mBottomRight.Y = y;
            mBottomRight.Z = z;
        }

        public Vector3 getBottomRightPoint()
        {
            return mBottomLeft;
        }

        //Not real exact since the bottoms aren't the same height
        public override float getVolume()
        {
            float b = Vector3.Distance(mBottomLeft, mBottomRight);
            float h = mTop.Y - Math.Min(mBottomRight.Y, mBottomLeft.Y);
            return ((1f/3f) * b * h);
        }

        public override void update(float elapsedSeconds)
        {
            Vector3 adjVelocity = mVelocity * elapsedSeconds;
            mTop += adjVelocity;
            mBottomLeft += adjVelocity;
            mBottomRight += adjVelocity;
        }

        public override UWB_BoundingVolume getBoundingVolume(eLevelofDetail lod)
        {
            Vector3 center = getCenter();
            float largestDistance = Math.Max(Math.Max(Vector3.Distance(mTop, center), Vector3.Distance(mBottomLeft, center)), Vector3.Distance(mBottomRight, center));
            mBounds.setCenterAndRadius(getCenter(), (largestDistance * 0.75f) / 2f);
            return mBounds;
        }
    }
}
