using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using NathanScott_Lib;

namespace UWBGL_XNA_Lib13
{
    public class UWB_WindowHandler: UWB_IWindowHandler
    {
        protected IntPtr m_hAttachedWindow;
		protected UWB_BoundingBox m_WCWindow;

		public UWB_WindowHandler()
		{
			m_WCWindow = new UWB_BoundingBox();
		}

        public override bool InitializeHandler(IntPtr hAttachedWindow)
        {
            if (!Control.FromHandle(hAttachedWindow).IsHandleCreated)
                return false;

            m_hAttachedWindow = hAttachedWindow;
            return true;
        }

        public bool hardwareToDevice(int hardwareX, int hardwareY, ref int deviceX, ref int deviceY)
        {
            if (!Control.FromHandle(m_hAttachedWindow).IsHandleCreated)
                return false;

            deviceX = hardwareX;
            deviceY = Control.FromHandle(m_hAttachedWindow).Height - hardwareY;
            return true;
        }

        public float getAspectRatio()
        {
            if (!Control.FromHandle(m_hAttachedWindow).IsHandleCreated)
                return 0.0f;

            Size size = Control.FromHandle(m_hAttachedWindow).Size;

            return (float)size.Width / (float)size.Height;
        }

		public void setWCWindow(ref UWB_BoundingBox w)
		{
			m_WCWindow = w;
		}

        public UWB_BoundingBox GetWCWWindow()
        {
            return m_WCWindow;
        }

		public virtual void loadW2NDCXform()
		{
			return;
		}

        // BEGIN SCOTT PORT FROM C++ LIBRARIES

        /// <summary>
        /// Gets the size of the device.
        /// </summary>
        /// <param name="width">The width.</param>
        /// <param name="height">The height.</param>
        public void GetDeviceSize(ref int width, ref int height)
        {
            Control ctrl = Control.FromHandle(m_hAttachedWindow);
            width = ctrl.Width;
            height = ctrl.Height;
        }

        /// <summary>
        /// Sets the size of the device.
        /// </summary>
        /// <param name="width">The width.</param>
        /// <param name="height">The height.</param>
        public virtual void SetDeviceSize(int width, int height)
        {
            Control ctrl = Control.FromHandle(m_hAttachedWindow);
            ctrl.Width = width;
            ctrl.Height = height;
        }

        /// <summary>
        /// Converts from device coordinate system to world coordinate system.
        /// </summary>
        /// <param name="dcX">The dc X.</param>
        /// <param name="dcY">The dc Y.</param>
        /// <param name="wcX">The wc X.</param>
        /// <param name="wcY">The wc Y.</param>
        public void DeviceToWorld(int dcX, int dcY, ref float wcX, ref float wcY)
        {
            int dcW = 0;
            int dcH = 0;
            GetDeviceSize(ref dcW, ref dcH);

            if (m_WCWindow != null)
            {
                Vector3 center = m_WCWindow.getCenter();

                wcX = center.X + ((m_WCWindow.getWidth() / dcW) * (dcX - (dcW / 2.0f)));
                wcY = center.Y + ((m_WCWindow.getHeight() / dcH) * (dcY - (dcH / 2.0f)));
            }
        }

        /// <summary>
        /// Converts from world coordinate system to device coordinate system.
        /// </summary>
        /// <param name="wcX">The wc X.</param>
        /// <param name="wcY">The wc Y.</param>
        /// <param name="dcX">The dc X.</param>
        /// <param name="dcY">The dc Y.</param>
        public void WorldToDevice(float wcX, float wcY, ref int dcX, ref int dcY)
        {
            int dcW = 0;
            int dcH = 0;
            GetDeviceSize(ref dcW, ref dcH);

            if (m_WCWindow != null)
            {
                Vector3 center = m_WCWindow.getCenter();

                dcX = N_Math.round((m_WCWindow.getWidth() / 2.0f) + ((dcW / m_WCWindow.getWidth()) * (-center.X + wcX)));
                dcY = N_Math.round((m_WCWindow.getHeight() / 2.0f) + ((dcH / m_WCWindow.getHeight()) * (-center.Y + wcY)));
            }
        }

        /// <summary>
        /// Draws the WC window.
        /// </summary>
        /// <param name="drawHelper">The draw helper.</param>
        public void DrawWCWindow(UWB_DrawHelper drawHelper)
        {
            drawHelper.resetAttributes();
            drawHelper.setShadeMode(eShadeMode.smFlat);
            drawHelper.setFillMode(eFillMode.fmWireframe);
            //TODO: Port over the UWB_Color class?  Do we need to?
            drawHelper.setColor1(new Microsoft.Xna.Framework.Color(N_Math.round(1 * 255.0), 0, 0));
            drawHelper.drawRectangle(m_WCWindow.getMin(), m_WCWindow.getMax());
        }
    }
}
