﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace UWBGL_XNA_Lib13
{
    public class Animation
    {
        private enum eAnimationState
        {
            Play,
            Pause,
        }

        private eAnimationState mState = eAnimationState.Play;
        private AnimationObject mAnimation = null;
        private bool mLoop = false;

        /// <summary>
        /// Constructor for an animation. Plays given animation.
        /// Make sure animation is not null.
        /// </summary>
        /// <param name="animation">animation to play</param>
        public Animation(AnimationObject animation)
        {
            mLoop = false;
            mState = eAnimationState.Play;
            mAnimation = animation;
        }

        /// <summary>
        /// Constructor for an animation. Plays given animation.
        /// Make sure animation is not null.
        /// </summary>
        /// <param name="animation">animation to play</param>
        public Animation(AnimationObject animation, bool loop)
        {
            mLoop = loop;
            mState = eAnimationState.Play;
            mAnimation = animation;
        }

        /// <summary>
        /// Plays the animation
        /// </summary>
        public void play()
        {
            mState = eAnimationState.Play;
        }

        /// <summary>
        /// Pauses the animation
        /// </summary>
        public void pause()
        {
            mState = eAnimationState.Pause;
        }

        /// <summary>
        /// Gets whether the animation is finished or not
        /// </summary>
        /// <returns>true if animation is finished, false if not</returns>
        public bool isFinished()
        {
            return mAnimation != null ? mAnimation.isFinished() : true;
        }

        /// <summary>
        /// Gets whether animation is looping or not
        /// </summary>
        /// <returns>true if animation is looping false if not</returns>
        public bool isLooping()
        {
            return mLoop;
        }

        public void startLooping()
        {
            mLoop = true;
        }

        public void stopLooping()
        {
            mLoop = false;
        }

        /// <summary>
        /// Plays the animation if it isn't paused.
        /// Should be called in tick to update the animation.
        /// </summary>
        /// <param name="elapsedSeconds">seconds passed since last tick</param>
        public void update(float elapsedSeconds)
        {
            if (mAnimation != null && mState != eAnimationState.Pause)
                mAnimation.update(elapsedSeconds);
        }
    }
}
