﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace UWBGL_XNA_Lib13
{
    public class AnimationInfo : AnimationObject
    {
        private float mTimeLeft = 0.0f;
        private float mAnimationTime = 0.0f;
        private UWB_SceneNode mSceneNode = null;
        UWB_XFormInfo mTargetXForm = null;
        UWB_XFormInfo mStartXForm = null;

        public AnimationInfo(float secondsToAnimate, UWB_SceneNode sceneNodeToAnimate, UWB_XFormInfo targetXform)
        {
            mAnimationTime = secondsToAnimate;
            mTimeLeft = mAnimationTime;
            mSceneNode = sceneNodeToAnimate;
            mTargetXForm = new UWB_XFormInfo(targetXform);
        }

        public override bool isFinished()
        {
            return (mTimeLeft <= 0.0f);
        }

        public override void update(float elapseSeconds)
        {
            if (mSceneNode != null && mTargetXForm != null)
            {
                //The first time we update get our starting state, so we can interpolate
                if (mStartXForm == null)
                    mStartXForm = new UWB_XFormInfo(mSceneNode.getXFormInfo());

                mTimeLeft = Math.Max(mTimeLeft - elapseSeconds, 0.0f);
                float amount = 1.0f - (mTimeLeft / mAnimationTime);
                UWB_XFormInfo xform = mSceneNode.getXFormInfo();

                xform.setTranslation(Vector3.Lerp(mStartXForm.getTranslation(), mTargetXForm.getTranslation(), amount));
                xform.setScale(Vector3.Lerp(mStartXForm.getScale(), mTargetXForm.getScale(), amount));
                xform.setPivot(Vector3.Lerp(mStartXForm.getPivot(), mTargetXForm.getPivot(), amount));
                xform.setRotationDegrees(MathHelper.Lerp(mStartXForm.getRotationInDegrees(), mTargetXForm.getRotationInDegrees(), amount));
            }
        }
    }
}
