using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib13
{
    public class UWB_XNADrawHelper : UWB_DrawHelper
    {

        public UWB_XNADrawHelper()
        {
      
        }

        public override bool drawPoint(Vector3 position)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawPoint call");

            // Draw points as rectangles
            VertexPositionColorTexture[] v = new VertexPositionColorTexture[4];
            float offset = mPointSize / 2f;

            for (int i = 0; i < 4; i++)
            {
                v[i] = new VertexPositionColorTexture();
                v[i].Color = mColor1;
            }

            v[0].Position = new Vector3(position.X - offset, position.Y + offset, 0);
            v[0].TextureCoordinate = new Vector2(0f, 0f);

            v[1].Position = new Vector3(position.X - offset, position.Y - offset, 0);
            v[1].TextureCoordinate = new Vector2(0f, 1f);

            v[2].Position = new Vector3(position.X + offset, position.Y + offset, 0);
            v[2].TextureCoordinate = new Vector2(1f, 0f);

            v[3].Position = new Vector3(position.X + offset, position.Y - offset, 0);
            v[3].TextureCoordinate = new Vector2(1f, 1f);

            if (device.RasterizerState.FillMode != FillMode.Solid)
            {
                RasterizerState s = new RasterizerState();
                s.CullMode = CullMode.None;
                s.FillMode = FillMode.Solid;
                device.RasterizerState = s;
            }

            device.DrawUserPrimitives(PrimitiveType.TriangleStrip, v, 0, 2);
            return true;
        }

        public override bool drawLine(Vector3 start, Vector3 end)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");

            VertexPositionColorTexture[] v = new VertexPositionColorTexture[2];

            Color flatColor = mColor1;

            v[0].Position = start;
            v[0].TextureCoordinate = new Vector2(0f, 0f);
            v[0].Color = flatColor;
            v[1].Position = end;
            v[1].TextureCoordinate = new Vector2(1f, 1f);
            if (mShadeMode == eShadeMode.smFlat)
                v[1].Color = flatColor;
            else
                v[1].Color = mColor2;

            device.DrawUserPrimitives(PrimitiveType.LineList, v, 0, 1);

            return true;
        }

        public override bool drawCircle(Vector3 center, float radius, float textureScale, bool useTextureAspectRatio)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");
            if (radius > 0)
            {
                int kNumTriangles = 40;
                if (mLod == eLevelofDetail.lodLow)
                    kNumTriangles = 10;
                else if (mLod == eLevelofDetail.lodMed)
                    kNumTriangles = 20;


                VertexPositionColorTexture[] v = new VertexPositionColorTexture[kNumTriangles * 3];

                float theta = (2.0f * (float)Math.PI) / kNumTriangles;

                v[0].Position = center;
                v[0].TextureCoordinate = new Vector2(0.5f, 0.5f);
                if (mShadeMode == eShadeMode.smGouraud)
                    v[0].Color = mColor2;
                else
                    v[0].Color = mColor1;

                float px = center.X + radius;
                float py = center.Y;
                Vector2 puv = new Vector2(1.0f, 0);

                for (int i = 0; i < kNumTriangles; i++)
                {
                    int offset = i * 3;
                    float x = center.X + (radius * (float)Math.Cos((i + 1) * theta));
                    float y = center.Y + (radius * (float)Math.Sin((i + 1) * theta));
                    float tu = 0.5f + (0.5f * (float)Math.Cos((i + 1) * theta));
                    float tv = 0.5f - (0.5f * (float)Math.Sin((i + 1) * theta));
                    v[offset].Position = v[0].Position;
                    v[offset].Color = v[0].Color;
                    v[offset].TextureCoordinate = v[0].TextureCoordinate;

                    v[offset + 1].Position = new Vector3(px, py, 0f);
                    v[offset + 1].Color = mColor1;
                    v[offset + 1].TextureCoordinate = puv;

                    v[offset + 2].Position = new Vector3(x, y, 0f);
                    v[offset + 2].Color = mColor1;
                    v[offset + 2].TextureCoordinate = new Vector2(tu, tv);

                    px = x;
                    py = y;
                    puv = v[offset + 2].TextureCoordinate;
                }

                //Scale everything
                if (textureScale != 1.0f)
                {
                    int count = kNumTriangles * 3;
                    for (int i = 0; i < count; i++)
                    {
                        v[i].TextureCoordinate.X *= textureScale;
                        v[i].TextureCoordinate.Y *= textureScale;
                    }
                }

                device.DrawUserPrimitives(PrimitiveType.TriangleList, v, 0, kNumTriangles);
            }
            return true;
        }

        public override bool drawRectangle(Vector3 min, Vector3 max, float textureScale, bool useTextureAspectRatio)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawRectangle call");

            VertexPositionColorTexture[] v = new VertexPositionColorTexture[12];
            Color flatColor = mColor1;

            float uMax = 1.0f;
            float vMax = 1.0f;

            if(useTextureAspectRatio)
            {
                /*
                Texture2D texture = UWB_XNAGraphicsSystem.m_TheAPI.MyEffect.Texture;
                if (texture != null)
                {
                    width = texture.Width;
                    height = texture.Height;
                }
                */
                float width = max.X - min.X;
                float height = max.Y - min.Y;
                if (width < height)
                {
                    vMax = 1.0f;
                    uMax = 1.0f / (height / width);
                }
                else if (width > height)
                {
                    vMax = 1.0f / (width / height);
                    uMax = 1.0f;
                }
            }
            uMax *= textureScale;
            vMax *= textureScale;


            if (mShadeMode == eShadeMode.smGouraud)
                v[0].Color = mColor2;
            else
                v[0].Color = flatColor;

            v[0].Position = Vector3.Divide((min + max), 2);
            v[0].TextureCoordinate = new Vector2(uMax/2.0f, vMax/2.0f);

            v[1].Position = new Vector3(max.X, min.Y, min.Z);
            v[1].TextureCoordinate = new Vector2(uMax, vMax);
            v[1].Color = flatColor;

            v[2].Position = max;
            v[2].TextureCoordinate = new Vector2(uMax, 0f);
            v[2].Color = flatColor;

            v[3] = v[0];
            v[4] = v[2];

            v[5].Position = new Vector3(min.X, max.Y, max.Z);
            v[5].TextureCoordinate = new Vector2(0f, 0f);
            v[5].Color = flatColor;

            v[6] = v[0];
            v[7] = v[5];
            v[8].Position = min;
            v[8].TextureCoordinate = new Vector2(0f, vMax);
            v[8].Color = flatColor;

            v[9] = v[0];
            v[10] = v[8];
            v[11] = v[1];

            device.DrawUserPrimitives(PrimitiveType.TriangleList, v, 0, 4);
            return true;
        }

        public override bool drawPolygon(Vector3 center, float radius)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");

            Color flatColor = mColor1;

            int kNumTriangles = 6;


            VertexPositionColor[] v = new VertexPositionColor[kNumTriangles * 3];

            float theta = (2.0f * (float)Math.PI) / kNumTriangles;

            v[0].Position = center;
            if (mShadeMode == eShadeMode.smGouraud)
                v[0].Color = mColor2;
            else
                v[0].Color = mColor1;

            float px = center.X + radius;
            float py = center.Y;

            for (int i = 0; i < kNumTriangles; i++)
            {
                int offset = i * 3;
                float x = center.X + (radius * (float)Math.Cos((i + 1) * theta));
                float y = center.Y + (radius * (float)Math.Sin((i + 1) * theta));
                v[offset].Position = v[0].Position;
                v[offset].Color = v[0].Color;
                v[offset + 1].Position = new Vector3(px, py, 0f);
                v[offset + 1].Color = mColor1;
                v[offset + 2].Position = new Vector3(x, y, 0f);
                v[offset + 2].Color = mColor1;

                px = x;
                py = y;
            }

            device.DrawUserPrimitives(PrimitiveType.TriangleList, v, 0, kNumTriangles);

            return true;
        }

        public override bool drawTriangle(Vector3 top, Vector3 bottomRight, Vector3 bottomLeft)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");

            VertexPositionColor[] v = new VertexPositionColor[3];

            v[0].Position = bottomLeft;
            if (mShadeMode == eShadeMode.smGouraud)
                v[0].Color = mColor2;
            else
                v[0].Color = mColor1;

            v[1].Position = top;
            v[1].Color = mColor1;

            v[2].Position = bottomRight;
            if (mShadeMode == eShadeMode.smGouraud)
                v[2].Color = mColor2;
            else
                v[2].Color = mColor1;


            device.DrawUserPrimitives(PrimitiveType.TriangleList, v, 0, 1);

            return true;
        }

        public override void setFillMode(eFillMode fillMode)
        {
            GraphicsDevice graphicsDevice = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            bool needToSetMode = false;
            FillMode newMode = FillMode.Solid;
            switch (fillMode)
            {
                case eFillMode.fmWireframe:
                case eFillMode.fmPoint:
                    needToSetMode = (graphicsDevice.RasterizerState.FillMode != FillMode.WireFrame);
                    newMode = FillMode.WireFrame;
                    break;
                case eFillMode.fmSolid:
                    needToSetMode = (graphicsDevice.RasterizerState.FillMode != FillMode.Solid);
                    break;
            }
            RasterizerState s = new RasterizerState();
            s.CullMode = CullMode.None;
            s.FillMode = newMode;
            graphicsDevice.RasterizerState = s;
        }

        public override bool accumulateModelTransform(
            Vector3 translation,
            Vector3 scale,
            float rotationRadians,
            Vector3 pivot)
        {
            if (UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice() == null)
                return false;

            Vector3 z_axis = new Vector3(0, 0, 1);
            mMatrixStack.TranslateLocal(translation);
            mMatrixStack.TranslateLocal(pivot);
            mMatrixStack.RotateAxisLocal(z_axis, rotationRadians);
            mMatrixStack.ScaleLocal(scale);
            mMatrixStack.TranslateLocal(-pivot);

            UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(mMatrixStack.Top());
            return true;

        }

        public override bool pushModelTransform()
        {
            if (UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice() == null)
                return false;

            mMatrixStack.Push();
            UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(mMatrixStack.Top());
            return true;
        }

        public override bool popModelTransform()
        {
            if (UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice() == null)
                return false;

            mMatrixStack.Pop();
            UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(mMatrixStack.Top());
            return true;
        }

        public override bool initializeModelTransform()
        {
            if (UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice() == null)
                return false;

            mMatrixStack.LoadIdentity();
            UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(mMatrixStack.Top());
            return true;
        }

        public override bool transformPoint(ref Vector3 point)
        {
            Vector3 pt = new Vector3(point.X, point.Y, point.Z);

            Matrix m = mMatrixStack.Top();
            point.X = pt.X * m.M11 + pt.Y * m.M21 + pt.Z * m.M31 + m.M41;
            point.Y = pt.X * m.M12 + pt.Y * m.M22 + pt.Z * m.M32 + m.M42;
            point.Z = pt.X * m.M13 + pt.Y * m.M23 + pt.Z * m.M33 + m.M43;
            return true;
        }

        public override bool enableBlending(bool on)
        {
            GraphicsDevice d = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            if (d != null)
            {
                mbBlendingEnabled = on;

                if (on)
                {
                    d.BlendState = BlendState.AlphaBlend;
                    // Same as doing the following ...
                    // d.BlendState.AlphaBlendFunction = BlendFunction.Add;
                    // d.BlendState.AlphaSourceBlend = Blend.SourceAlpha;
                    // d.BlendState.AlphaDestinationBlend = Blend.InverseSourceAlpha;
                }
                else
                {
                    d.BlendState = BlendState.Opaque;
                }
            }
            return true;
        }

        public override bool enableTexture(bool on)
        {
            mbTexturingEnabled = on;

            if (mbTexturingEnabled)
            {
                if (mTexFileName != "")
                    UWB_XNAGraphicsSystem.m_TheAPI.ActivateTexture(mTexFileName);
            }
            else
            {
                UWB_XNAGraphicsSystem.m_TheAPI.DeactivateTexture();
            }
            return !on;
        }

    }
}
